import os
import sys
from tempfile import mktemp


class CaptureStream(object):
    """A context manager which captures any errors on a given stream (like
    sys.stderr).  The stream is captured and the outputs can be used.

    We treat sys.stderr and stdout specially as very often these are
    overridden by nose or IPython.  We always wrap the underlying file
    descriptors in this case as this is the intent of this context manager.

    This is somewhat based on this question:
        http://stackoverflow.com/questions/7018879/disabling-output-when-compiling-with-distutils

    Examples
    --------

    See the tests in tests/test_capture_stream.py for example usage.
    """

    def __init__(self, stream=sys.stderr):
        self.stream = stream
        if stream is sys.stderr:
            self.fileno = 2
        elif stream is sys.stdout:
            self.fileno = 1
        else:
            self.fileno = stream.fileno()
        self.orig_stream = None
        self.tmp_stream = None
        self.tmp_path = ''
        self._cached_output = None

    def __enter__(self):
        self.orig_stream = os.dup(self.fileno)
        self.tmp_path = mktemp()
        self.tmp_stream = open(self.tmp_path, 'w+')
        os.dup2(self.tmp_stream.fileno(), self.fileno)
        return self

    def __exit__(self, type, value, tb):
        if self.orig_stream is not None:
            os.dup2(self.orig_stream, self.fileno)
        if self.tmp_stream is not None:
            self._cache_output()
            self.tmp_stream.close()
            os.remove(self.tmp_path)

    def _cache_output(self):
        if self._cached_output is not None:
            return
        tmp_stream = self.tmp_stream
        result = ''
        if tmp_stream is not None:
            tmp_stream.flush()
            tmp_stream.seek(0)
            result = tmp_stream.read()
        self._cached_output = result

    def get_output(self):
        """Return the captured output.
        """
        if self._cached_output is None:
            self._cache_output()
        return self._cached_output

class CaptureMultipleStreams(object):
    """This lets one capture multiple streams together.
    """
    def __init__(self, streams=None):
        streams = (sys.stdout, sys.stderr) if streams is None else streams
        self.streams = streams
        self.captures = [CaptureStream(x) for x in streams]
    def __enter__(self):
        for capture in self.captures:
            capture.__enter__()
        return self
    def __exit__(self, type, value, tb):
        for capture in self.captures:
            capture.__exit__(type, value, tb)
    def get_output(self):
        return tuple(x.get_output() for x in self.captures)




import ast
from textwrap import dedent
import unittest

from pysph.base.ast_utils import (get_assigned, get_aug_assign_symbols, get_symbols,
    has_node, has_return)


class TestASTUtils(unittest.TestCase):
    def test_get_symbols(self):
        code = '''
        x = 1
        d_x[d_idx] += s_x[s_idx]
        '''
        tree = ast.parse(dedent(code))
        result = list(get_symbols(tree))
        result.sort()
        expect = ['d_idx', 'd_x', 's_idx', 's_x', 'x']
        self.assertEqual(result, expect)

        # Test if it parses with the code itself instead of a tree.
        result = list(get_symbols(dedent(code)))
        result.sort()
        self.assertEqual(result, expect)

        result = list(get_symbols(tree, ctx=ast.Store))
        result.sort()
        self.assertEqual(result, ['x'])

    def test_get_aug_assign_symbols(self):
        code = '''
        x = 1
        '''
        result = list(get_aug_assign_symbols(dedent(code)))
        result.sort()
        expect = []
        self.assertEqual(result, expect)

        code = '''
        x += 1
        d_x[d_idx] += s_x[s_idx]
        '''
        result = list(get_aug_assign_symbols(dedent(code)))
        result.sort()
        expect = ['d_x', 'x']
        self.assertEqual(result, expect)

    def test_has_return(self):
        code = dedent('''
                x = 1
                ''')
        self.assertFalse(has_return(code))
        code = dedent('''
                def f():
                    pass
                ''')
        self.assertFalse(has_return(code))
        code = dedent('''
                def f(x):
                    return x+1
                ''')
        self.assertTrue(has_return(code))

    def test_has_node(self):
        code = dedent('''
                x = 1
                ''')
        self.assertFalse(has_node(code, (ast.Return, ast.AugAssign)))
        code = dedent('''
                def f():
                    pass
                ''')
        self.assertTrue(has_node(code, (ast.AugAssign, ast.FunctionDef)))

    def test_assigned_values(self):
        code = dedent('''
            u[0] = 0.0
            x = 1
            y = sin(x)*theta
            z += 1
            ''')
        assigned = list(sorted(get_assigned(code)))
        # sin or theta should not be detected.
        expect = ['u', 'x', 'y', 'z']
        self.assertEqual(assigned, expect)

    def test_assigned_tuple_expansion(self):
        code = dedent('''
            u, v = 0.0, 1.0
            [x, y] = 0.0, 1.0
            ''')
        assigned = list(sorted(get_assigned(code)))
        expect = ['u', 'v', 'x', 'y']
        self.assertEqual(assigned, expect)


if __name__ == '__main__':
    unittest.main()



from pysph.base.ext_module import ExtModule
from pysph.sph.integrator_cython_helper import IntegratorCythonHelper
from pysph.sph.acceleration_eval_cython_helper import AccelerationEvalCythonHelper


###############################################################################
class SPHCompiler(object):
    def __init__(self, acceleration_eval, integrator):
        self.acceleration_eval = acceleration_eval
        self.acceleration_eval_helper = AccelerationEvalCythonHelper(
            self.acceleration_eval
        )
        self.integrator = integrator
        self.integrator_helper = IntegratorCythonHelper(
            integrator, self.acceleration_eval_helper
        )
        self.ext_mod = None
        self.module = None

    #### Public interface. ####################################################
    def compile(self):
        """Compile the generated code to an extension module and
        setup the objects that need this by calling their setup_compiled_module.
        """
        if self.ext_mod is not None:
            return
        code = self._get_code()
        # Note, we do not add carray or particle_array as nnps would have
        # been rebuilt anyway if they changed.
        depends = ["pysph.base.nnps"]
        self.ext_mod = ExtModule(code, verbose=True, depends=depends)
        mod = self.ext_mod.load()
        self.module = mod

        self.acceleration_eval_helper.setup_compiled_module(mod)
        cython_a_eval = self.acceleration_eval.c_acceleration_eval
        if self.integrator is not None:
            self.integrator_helper.setup_compiled_module(mod, cython_a_eval)

    #### Private interface. ####################################################
    def _get_code(self):
        main = self.acceleration_eval_helper.get_code()
        integrator_code = self.integrator_helper.get_code()
        return main + integrator_code



"""Colliding Elastic Balls. (10 minutes)
"""

import numpy

# SPH equations
from pysph.sph.equation import Group
from pysph.sph.basic_equations import IsothermalEOS, ContinuityEquation, MonaghanArtificialViscosity,\
     XSPHCorrection, VelocityGradient2D
from pysph.sph.solid_mech.basic import MomentumEquationWithStress2D, HookesDeviatoricStressRate2D,\
    MonaghanArtificialStress

from pysph.base.utils import get_particle_array
from pysph.base.kernels import CubicSpline
from pysph.solver.application import Application
from pysph.solver.solver import Solver
from pysph.sph.integrator import PECIntegrator
from pysph.sph.integrator_step import SolidMechStep

def get_K(G, nu):
    ''' Get the bulk modulus from shear modulus and Poisson ratio '''
    return 2.0*G*(1+nu)/(3*(1-2*nu))

def add_properties(pa, *props):
    for prop in props:
        pa.add_property(name=prop)


# constants
E = 1e7
nu = 0.3975
G = E/(2.0*(1+nu))
K = get_K(G, nu)
rho0 = 1.0
c0 = numpy.sqrt(K/rho0)

dx = 0.0005
hdx = 1.5

# geometry
ri = 0.03
ro = 0.04

class Rings(Application):
    def create_particles(self):
        #x,y = numpy.mgrid[-1.05:1.05+1e-4:dx, -0.105:0.105+1e-4:dx]
        spacing = 0.041 # spacing = 2*5cm

        x,y = numpy.mgrid[-ro:ro:dx, -ro:ro:dx]
        x = x.ravel()
        y = y.ravel()

        d = (x*x+y*y)
        keep = numpy.flatnonzero((ri*ri<=d) * (d<ro*ro))
        x = x[keep]
        y = y[keep]

        x = numpy.concatenate([x-spacing,x+spacing])
        y = numpy.concatenate([y,y])

        print('Ellastic Collision with %d particles'%(x.size))
        print("Shear modulus G = %g, Young's modulus = %g, Poisson's ratio =%g"%(G,E,nu))

        #print bdry, numpy.flatnonzero(bdry)
        m = numpy.ones_like(x)*dx*dx
        h = numpy.ones_like(x)*hdx*dx
        rho = numpy.ones_like(x)
        z = numpy.zeros_like(x)

        p = 0.5*1.0*100*100*(1 - (x**2 + y**2))

        cs = numpy.ones_like(x) * 10000.0

        # u is set later
        v = z
        u_f = 0.059

        p *= 0
        h *= 1

        # create the particle array
        pa = get_particle_array(
            name="solid", x=x+spacing, y=y, m=m, rho=rho, h=h,
            p=p, cs=cs, u=z, v=v)

        pa.cs[:] = c0
        pa.u = pa.cs*u_f*(2*(x<0)-1)

        # add requisite properties

        # sound speed etc.
        add_properties(pa, 'cs', 'e' )

        # velocity gradient properties
        add_properties(pa, 'v00', 'v01', 'v10', 'v11')

        # artificial stress properties
        add_properties(pa, 'r00', 'r01', 'r02', 'r11', 'r12', 'r22')

        # deviatoric stress components
        add_properties(pa, 's00', 's01', 's02', 's11', 's12', 's22')

        # deviatoric stress accelerations
        add_properties(pa, 'as00', 'as01', 'as02', 'as11', 'as12', 'as22')

        # deviatoric stress initial values
        add_properties(pa, 's000', 's010', 's020', 's110', 's120', 's220')

        # standard acceleration variables
        add_properties(pa, 'arho', 'au', 'av', 'aw', 'ax', 'ay', 'az', 'ae')

        # initial values
        add_properties(pa, 'rho0', 'u0', 'v0', 'w0', 'x0', 'y0', 'z0', 'e0')

        # load balancing properties
        pa.set_lb_props( list(pa.properties.keys()) )

        return [pa,]

    def create_solver(self):
        kernel = CubicSpline(dim=2)
        self.wdeltap = kernel.kernel(rij=dx, h=hdx*dx)

        integrator = PECIntegrator(solid=SolidMechStep())

        solver = Solver(kernel=kernel, dim=2, integrator=integrator)

        dt = 1e-8
        tf = 5e-5
        solver.set_time_step(dt)
        solver.set_final_time(tf)
        solver.set_print_freq(500)
        return solver

    def create_equations(self):
        equations = [

            # Properties computed set from the current state
            Group(
                equations=[
                    # p
                    IsothermalEOS(dest='solid', sources=None,
                                  rho0=rho0, c0=c0, p0=0.0),

                    # vi,j : requires properties v00, v01, v10, v11
                    VelocityGradient2D(dest='solid', sources=['solid',]),

                    # rij : requires properties r00, r01, r02, r11, r12, r22,
                    #                           s00, s01, s02, s11, s12, s22
                    MonaghanArtificialStress(
                     dest='solid', sources=None, eps=0.3),
                    ],
                ),

            # Acceleration variables are now computed
            Group(
                equations=[

                    # arho
                    ContinuityEquation(dest='solid', sources=['solid',]),

                    # au, av
                    MomentumEquationWithStress2D(
                        dest='solid', sources=['solid',], n=4, wdeltap=self.wdeltap),

                    # au, av
                    MonaghanArtificialViscosity(
                        dest='solid', sources=['solid',], alpha=1.0, beta=1.0),

                    # a_s00, a_s01, a_s11
                    HookesDeviatoricStressRate2D(
                        dest='solid', sources=None, shear_mod=G),

                    # ax, ay, az
                    XSPHCorrection(
                        dest='solid', sources=['solid',], eps=0.5),

                    ]

                ) # End Acceleration Group

        ] # End Group list
        return equations


if __name__ == '__main__':
    app = Rings()
    app.run()



"""Example for the Zoltan Distributed data directory"""
from zoltan cimport _check_error

cdef class Zoltan_DD:
    """A Zoltan Distributed data directory is used as a parallel hash map.

    The hash map is implemented as keys of type ZOLTAN_ID_TYPE which
    can be global indices for the objects being partitioned for
    example. Each entry is owned by a processor and has associated
    with it, additional user data.

    """
    def __init__(self, mpi.Comm comm):
        """Initialize the Zoltan DD (ZDD)

        Parameters:
        -----------

        comm : mpi.Comm
            MPI communicator ( COMM_WORLD )

        All processes must instantiate a copy of the Zoltan DD. A
        pointer to the ZDD is created and stored for furhter
        calls. Upon destruction, Zoltan_Destroy is called to get rid
        of the allocated ZDD

        """
        self.comm = comm.ob_mpi
        ierr = Zoltan_DD_Create(&self.dd, self.comm, 1, 1, 0, 0, 0)

        _check_error( ierr )

    def Zoltan_DD_Update(self, UIntArray gid, IntArray part):
        """Populate the ZDD with some data.

        Parameters:
        ------------

        gid : UIntArray
            Global indices for the keys in the hash map

        part : UIntArray
            Partition/Processor which owns the data

        The ZDD can store additional user data and local indices with
        each hash entry. To skip these, we pass in Cython NULL
        pointers as recommended by the Zoltan user guide.

        """
        cdef int nentries = gid.length
        ierr = Zoltan_DD_Update(self.dd, gid.data, NULL, NULL, part.data, nentries)

        _check_error( ierr )

    def Zoltan_DD_Find(self, UIntArray gid, IntArray part, IntArray own,
                       object lid=None, data=None):
        """Look up ownership and partitions for a given entry.

        Parameters:
        -----------

        gid : UIntArray (in)
            Global indices for requested data

        part : IntArray (out)
            Partition/Processor to which the entry belongs

        own : IntArray (out)
            Partition/Processor to which the associated data belongs.

        lid, data : None (not used)

        """
        cdef int count = gid.length
        cdef int ierr

        # resize the own and part arrays
        own.resize( count )
        part.resize( count )

        ierr = Zoltan_DD_Find( self.dd, gid.data, NULL, NULL,
                               part.data, count, own.data )
        _check_error( ierr )

    def Zoltan_DD_Print(self):
        """Print the contents of the DD"""
        Zoltan_DD_Print( self.dd )

    def __dealloc__(self):
        """Boom!"""
        Zoltan_DD_Destroy( &self.dd )



import numpy as np

from pysph.base.kernels import CubicSpline
from pysph.base.particle_array import ParticleArray

from pysph.sph.equation import Equation
from pysph.sph.integrator_step import IntegratorStep

from pysph.sph.integrator import EulerIntegrator
from pysph.solver.application import Application
from pysph.solver.solver import Solver

def create_particles():
    x = np.linspace(0, 10, 10)
    m = np.ones_like(x)
    y = np.zeros_like(x)
    z = np.zeros_like(x)
    h = np.ones_like(x)*0.2

    fluid = ParticleArray(name='fluid', x=x, y=y, z=z, m=m, h=h)
    fluid.add_constant('total_mass', 0.0)
    return [fluid]

class TotalMass(Equation):
    def reduce(self, dst):
        # Use the dst.array as we want to pass the real particles to do the sum
        # passing the carray will send all the particles including ghosts
        # which is incorrect.
        m = serial_reduce_array(dst.array.m, op='sum')
        dst.total_mass[0] = parallel_reduce_array(m, op='sum')

class DummyStepper(IntegratorStep):
    def initialize(self):
        pass

    def stage1(self):
        pass

# Create the application.
app = Application()

dim = 1
# Create the kernel
kernel = CubicSpline(dim=dim)

# Create the integrator.
integrator = EulerIntegrator(fluid=DummyStepper())

solver = Solver(kernel=kernel, dim=dim, integrator=integrator)
solver.set_time_step(0.1)
solver.set_final_time(0.1)

equations = [TotalMass(dest='fluid', sources=['fluid'])]
app.setup(solver=solver, equations=equations,
          particle_factory=create_particles)
# There is no need to write any output as the test below
# computes the total mass.
solver.set_disable_output(True)
app.run()

fluid = solver.particles[0]
err = fluid.total_mass[0] - 10.0
assert abs(err) < 1e-16, "Error: %s"%err



"""Two-dimensional Shocktube problem. (10 mins)

The density is assumed to be uniform and the shocktube problem is
defined by the pressure jump. The pressure jump of 10^5 (pl = 1000.0,
pr = 0.01) corresponds to the Woodward and Colella strong shock or
blastwave problem.

"""

# NumPy and standard library imports
import numpy

from pysph.base.nnps import DomainManager
from pysph.base.utils import get_particle_array as gpa
from pysph.solver.application import Application

from pysph.sph.scheme import GasDScheme

# PySPH tools
from pysph.tools import uniform_distribution as ud

# Numerical constants
dim = 2
gamma = 1.4
gamma1 = gamma - 1.0

# solution parameters
dt = 7.5e-6
tf = 0.0075

# domain size
xmin = -0.75; xmax = 0.75
dx = 0.01
ymin = 0; ymax = 30*dx

# scheme constants
alpha1 = 1.0
alpha2 = 0.1
beta = 2.0
kernel_factor = 1.2
h0 = kernel_factor*dx


class ShockTube2D(Application):
    def create_domain(self):
        return DomainManager(
            xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
            periodic_in_x=True, periodic_in_y=True)

    def create_particles(self):
        global dx
        data = ud.uniform_distribution_cubic2D(dx, xmin, xmax, ymin, ymax)

        x = data[0]; y = data[1]
        dx = data[2]; dy = data[3]

        # volume estimate
        volume = dx*dy

        # indices on either side of the initial discontinuity
        right_indices = numpy.where( x > 0.0 )[0]

        # density is uniform
        rho = numpy.ones_like(x)

        # pl = 100.0, pr = 0.1
        p = numpy.ones_like(x) * 1000.0
        p[right_indices] = 0.01

        # const h and mass
        h = numpy.ones_like(x) * h0
        m = numpy.ones_like(x) * volume * rho

        # thermal energy from the ideal gas EOS
        e = p/(gamma1*rho)

        fluid = gpa(name='fluid', x=x, y=y, rho=rho, p=p, e=e, h=h, m=m,
                    h0=h.copy())
        self.scheme.setup_properties([fluid])

        print("2D Shocktube with %d particles"%(fluid.get_number_of_particles()))

        return [fluid,]

    def create_scheme(self):
        s = GasDScheme(
            fluids=['fluid'], solids=[], dim=dim, gamma=gamma,
            kernel_factor=kernel_factor, alpha1=alpha1, alpha2=alpha2,
            beta=beta
        )
        s.configure_solver(dt=dt, tf=tf, adaptive_timestep=False, pfreq=50)
        return s

if __name__ == '__main__':
    app = ShockTube2D()
    app.run()



"""Test if the mpi_reduce_array function works correctly.
"""

import mpi4py.MPI as mpi
import numpy as np

from pysph.base.reduce_array import serial_reduce_array, mpi_reduce_array

comm = mpi.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()
n = 5
data = np.ones(n)*(rank + 1)

full_data = []
for i in range(size):
    full_data = np.concatenate([full_data, np.ones(n)*(i+1)])

for op in ('sum', 'prod', 'min', 'max'):
    serial_data = serial_reduce_array(data, op)
    result = mpi_reduce_array(serial_data, op)
    expect = getattr(np, op)(full_data)
    msg = "For op %s: Expected %s, got %s"%(op, expect, result)
    assert expect == result, msg



# See PEP 440 for more on suitable version numbers.
__version__ = '1.0a4.dev0'

# Utility functions to determine if Zoltan/MPI are available.
_has_mpi = None
_has_zoltan = None
_in_parallel = None


def has_mpi():
    """Return True if mpi4py is available.
    """
    global _has_mpi
    if _has_mpi is None:
        _has_mpi = True
        try:
            import mpi4py
        except ImportError:
            _has_mpi = False
    return _has_mpi

def has_zoltan():
    """Return True if zoltan is available.
    """
    global _has_zoltan
    if _has_zoltan is None:
        _has_zoltan = True
        try:
            from pyzoltan.core import zoltan
        except ImportError:
            _has_zoltan = False
    return _has_zoltan

def in_parallel():
    """Return true if we're running with MPI and Zoltan support
    """
    global _in_parallel
    if _in_parallel is None:
        _in_parallel = has_mpi() and has_zoltan()

    return _in_parallel

# Utility function to determine the possible output files

_has_h5py = None
_has_pyvisfile = None
_has_tvtk = None


def has_h5py():
    """Return True if h5py is available.
    """
    global _has_h5py
    if _has_h5py is None:
        _has_h5py = True
        try:
            import h5py
        except ImportError:
            _has_h5py = False
    return _has_h5py

def has_tvtk():
    """Return True if tvtk is available.
    """
    global _has_tvtk
    if _has_tvtk is None:
        _has_tvtk = True
        try:
            import tvtk
        except ImportError:
            _has_tvtk = False
    return _has_tvtk

def has_pyvisfile():
    """Return True if pyvisfile is available.
    """
    global _has_pyvisfile
    if _has_pyvisfile is None:
        _has_pyvisfile = True
        try:
            import pyvisfile
        except ImportError:
            _has_pyvisfile = False
    return _has_pyvisfile




"""A particle viewer using Mayavi.

This code uses the :py:class:`MultiprocessingClient` solver interface to
communicate with a running solver and displays the particles using
Mayavi.  It can also display a list of supplied files or a directory.
"""

from functools import reduce
import glob
import sys
import math
import numpy
import os
import os.path

if not os.environ.get('ETS_TOOLKIT'):
    # Set the default toolkit to qt4 unless the user has explicitly
    # set the default manually via the env var.
    try:
        from traits.etsconfig.api import ETSConfig
    except ImportError:
        from enthought.etsconfig.api import ETSConfig
    ETSConfig.toolkit = 'qt4'

try:
    from traits.api import (Any, Array, Dict, HasTraits, Instance,
        on_trait_change, List, Str, Int, Range, Float, Bool, Button,
        Directory, Event, Password, Property, cached_property)
    from traitsui.api import (View, Item, Group, Handler, HSplit, ListEditor,
        EnumEditor, TitleEditor, HGroup, ShellEditor, VSplit)
    from mayavi.core.api import PipelineBase
    from mayavi.core.ui.api import (MayaviScene, SceneEditor, MlabSceneModel)
    from pyface.timer.api import Timer, do_later
    from tvtk.api import tvtk
    from tvtk.array_handler import array2vtk
except ImportError:
    from enthought.traits.api import (Any, Array, Dict, HasTraits,
        Instance, on_trait_change, List, Str, Int, Range, Float, Bool, Button,
        Directory, Event, Password, Property, cached_property)
    from enthought.traits.ui.api import (View, Item, Group, Handler,
        HSplit, ListEditor, EnumEditor, TitleEditor, HGroup, ShellEditor,
        VSplit)
    from enthought.mayavi.core.api import PipelineBase
    from enthought.mayavi.core.ui.api import (MayaviScene, SceneEditor,
                                              MlabSceneModel)
    from enthought.pyface.timer.api import Timer, do_later
    from enthought.tvtk.api import tvtk
    from enthought.tvtk.array_handler import array2vtk

from pysph.base.particle_array import ParticleArray
from pysph.solver.solver_interfaces import MultiprocessingClient
from pysph.solver.utils import load, dump, output_formats
from pysph.solver.utils import remove_irrelevant_files, _sort_key
from pysph.tools.interpolator import (get_bounding_box, get_nx_ny_nz,
    Interpolator)


import logging
logger = logging.getLogger()

def set_arrays(dataset, particle_array):
    """ Code to add all the arrays to a dataset given a particle array."""
    props = set(particle_array.properties.keys())
    # Add the vector data.
    vec = numpy.empty((len(particle_array.x), 3), dtype=float)
    vec[:,0] = particle_array.u
    vec[:,1] = particle_array.v
    vec[:,2] = particle_array.w
    va = tvtk.to_tvtk(array2vtk(vec))
    va.name = 'velocity'
    dataset.data.point_data.add_array(vec)
    # Now add the scalar data.
    scalars = props - set(('u', 'v', 'w'))
    for sc in scalars:
        arr = particle_array.get(sc)
        va = tvtk.to_tvtk(array2vtk(arr))
        va.name = sc
        dataset.data.point_data.add_array(va)
    dataset._update_data()

def glob_files(fname):
    """Glob for all similar files given one of them.

    This assumes that the files are of the form *_[0-9]*.*.
    """
    fbase = fname[:fname.rfind('_')+1]
    ext = fname[fname.rfind('.'):]
    return glob.glob("%s*%s"%(fbase, ext))

def sort_file_list(files):
    """Given a list of input files, sort them in serial order, in-place.
    """
    files[:] = remove_irrelevant_files(files)
    files.sort(key=_sort_key)
    return files


##############################################################################
# `InterpolatorView` class.
##############################################################################
class InterpolatorView(HasTraits):

    # The bounds on which to interpolate.
    bounds = Array(cols=3, dtype=float,
                   desc='spatial bounds for the interpolation '\
                        '(xmin, xmax, ymin, ymax, zmin, zmax)')

    # The number of points to interpolate onto.
    num_points = Int(100000, enter_set=True, auto_set=False,
                     desc='number of points on which to interpolate')

    # The particle arrays to interpolate from.
    particle_arrays = List

    # The scalar to interpolate.
    scalar = Str('rho', desc='name of the active scalar to view')

    # Sync'd trait with the scalar lut manager.
    show_legend = Bool(False, desc='if the scalar legend is to be displayed')

    # Enable/disable the interpolation
    visible = Bool(False, desc='if the interpolation is to be displayed')

    # A button to use the set bounds.
    set_bounds = Button('Set Bounds')

    # A button to recompute the bounds.
    recompute_bounds = Button('Recompute Bounds')

    #### Private traits. ######################################################

    # The interpolator we are a view for.
    interpolator = Instance(Interpolator)

    # The mlab plot for this particle array.
    plot = Instance(PipelineBase)

    scalar_list = List

    scene = Instance(MlabSceneModel)

    source = Instance(PipelineBase)

    _arrays_changed = Bool(False)

    #### View definition ######################################################
    view = View(Item(name='visible'),
                Item(name='scalar',
                     editor=EnumEditor(name='scalar_list')
                    ),
                Item(name='num_points'),
                Item(name='bounds'),
                Item(name='set_bounds', show_label=False),
                Item(name='recompute_bounds', show_label=False),
                Item(name='show_legend'),
                )

    #### Private protocol  ###################################################
    def _change_bounds(self):
        interp = self.interpolator
        if interp is not None:
            interp.set_domain(self.bounds, self.interpolator.shape)
            self._update_plot()

    def _setup_interpolator(self):
        if self.interpolator is None:
            interpolator = Interpolator(
                self.particle_arrays, num_points=self.num_points
            )
            self.bounds = interpolator.bounds
            self.interpolator = interpolator
        else:
            if self._arrays_changed:
                self.interpolator.update_particle_arrays(self.particle_arrays)
                self._arrays_changed = False

    #### Trait handlers  #####################################################
    def _particle_arrays_changed(self, pas):
        if len(pas) > 0:
            all_props = reduce(set.union,
                               [set(x.properties.keys()) for x in pas])
        else:
            all_props = set()
        self.scalar_list = list(all_props)
        self._arrays_changed = True
        self._update_plot()

    def _num_points_changed(self, value):
        interp = self.interpolator
        if interp is not None:
            bounds = self.interpolator.bounds
            shape = get_nx_ny_nz(value, bounds)
            interp.set_domain(bounds, shape)
            self._update_plot()

    def _recompute_bounds_fired(self):
        bounds = get_bounding_box(self.particle_arrays)
        self.bounds = bounds
        self._change_bounds()

    def _set_bounds_fired(self):
        self._change_bounds()

    def _bounds_default(self):
        return [0, 1, 0, 1, 0, 1]

    @on_trait_change('scalar, visible')
    def _update_plot(self):
        if self.visible:
            mlab = self.scene.mlab
            self._setup_interpolator()
            interp = self.interpolator
            prop = interp.interpolate(self.scalar)
            if self.source is None:
                src = mlab.pipeline.scalar_field(
                    interp.x, interp.y, interp.z, prop
                )
                self.source = src
            else:
                self.source.mlab_source.reset(
                    x=interp.x, y=interp.y, z=interp.z, scalars=prop
                )
            src = self.source

            if self.plot is None:
                if interp.dim == 3:
                    plot = mlab.pipeline.scalar_cut_plane(src)
                else:
                    plot = mlab.pipeline.surface(src)
                self.plot = plot
                scm = plot.module_manager.scalar_lut_manager
                scm.set(show_legend=self.show_legend,
                        use_default_name=False,
                        data_name=self.scalar)
                self.sync_trait('show_legend', scm, mutual=True)
            else:
                self.plot.visible = True
                scm = self.plot.module_manager.scalar_lut_manager
                scm.data_name = self.scalar
        else:
            if self.plot is not None:
                self.plot.visible = False


##############################################################################
# `ParticleArrayHelper` class.
##############################################################################
class ParticleArrayHelper(HasTraits):
    """
    This class manages a particle array and sets up the necessary
    plotting related information for it.
    """

    # The particle array we manage.
    particle_array = Instance(ParticleArray)

    # The name of the particle array.
    name = Str

    # Current time.
    time = Float(0.0)

    # The active scalar to view.
    scalar = Str('rho', desc='name of the active scalar to view')

    # The mlab plot for this particle array.
    plot = Instance(PipelineBase)

    # List of available scalars in the particle array.
    scalar_list = List(Str)

    scene = Instance(MlabSceneModel)

    # Sync'd trait with the scalar lut manager.
    show_legend = Bool(False, desc='if the scalar legend is to be displayed')

    # Sync'd trait with the dataset to turn on/off visibility.
    visible = Bool(True, desc='if the particle array is to be displayed')

    # Show the time of the simulation on screen.
    show_time = Bool(False, desc='if the current time is displayed')

    # Do we show the hidden arrays?
    show_hidden_arrays = Bool(False,
                              desc='if hidden arrays are to be listed')

    # Private attribute to store the Text module.
    _text = Instance(PipelineBase)

    # Extra scalars to show.  These will be added and saved to the data if
    # needed.
    extra_scalars = List(Str)

    # Set to True when the particle array is updated with a new property say.
    updated = Event

    ########################################
    # View related code.
    view = View(Item(name='name',
                     show_label=False,
                     editor=TitleEditor()),
                Group(
                      Item(name='visible'),
                      Item(name='show_hidden_arrays'),
                      Item(name='scalar',
                           editor=EnumEditor(name='scalar_list')
                          ),
                      Item(name='show_legend'),
                      Item(name='show_time'),
                      ),
                )

    #######  Private protocol ############################################
    def _add_vmag(self, pa):
        if 'vmag' not in pa.properties:
            if 'vmag2' in pa.output_property_arrays:
                vmag = numpy.sqrt(pa.vmag2)
            else:
                vmag = numpy.sqrt(pa.u**2 + pa.v**2 + pa.w**2)
            pa.add_property(name='vmag', data=vmag)
            if len(pa.output_property_arrays) > 0:
                # We do not call add_output_arrays when the default is empty
                # as if it is empty, all arrays are saved anyway. However,
                # adding just vmag in this case will mean that when the
                # particle array is saved it will only save vmag!  This is
                # not what we want, hence we add vmag *only* if the
                # output_property_arrays is non-zero length.
                pa.add_output_arrays(['vmag'])
            self.updated = True

    def _get_scalar(self, pa, scalar):
        """Return the requested scalar from the given particle array.
        """
        if scalar in self.extra_scalars:
            method_name = '_add_' + scalar
            method = getattr(self, method_name)
            method(pa)
        return getattr(pa, scalar)

    #######  Traits handlers #############################################
    def _particle_array_changed(self, pa):
        self.name = pa.name
        # Setup the scalars.
        self._show_hidden_arrays_changed(self.show_hidden_arrays)

        # Update the plot.
        x, y, z = pa.x, pa.y, pa.z
        s = self._get_scalar(pa, self.scalar)
        p = self.plot
        mlab = self.scene.mlab
        if p is None:
            src = mlab.pipeline.scalar_scatter(x, y, z, s)
            p = mlab.pipeline.glyph(src, mode='point', scale_mode='none')
            p.actor.property.point_size = 3
            scm = p.module_manager.scalar_lut_manager
            scm.set(show_legend=self.show_legend,
                    use_default_name=False,
                    data_name=self.scalar)
            self.sync_trait('visible', p.mlab_source.m_data,
                             mutual=True)
            self.sync_trait('show_legend', scm, mutual=True)
            #set_arrays(p.mlab_source.m_data, pa)
            self.plot = p
        else:
            if len(x) == len(p.mlab_source.x):
                p.mlab_source.set(x=x, y=y, z=z, scalars=s)
            else:
                p.mlab_source.reset(x=x, y=y, z=z, scalars=s)

        # Setup the time.
        self._show_time_changed(self.show_time)

    def _scalar_changed(self, value):
        p = self.plot
        if p is not None:
            p.mlab_source.scalars = self._get_scalar(
                self.particle_array, value
            )
            p.module_manager.scalar_lut_manager.data_name = value

    def _show_hidden_arrays_changed(self, value):
        pa = self.particle_array
        sc_list = pa.properties.keys()
        if value:
            self.scalar_list = sorted(set(sc_list + self.extra_scalars))
        else:
            self.scalar_list = sorted(
                set(
                    [x for x in sc_list if not x.startswith('_')] +
                    self.extra_scalars
                )
            )

    def _show_time_changed(self, value):
        txt = self._text
        mlab = self.scene.mlab
        if value:
            if txt is not None:
                txt.visible = True
            elif self.plot is not None:
                mlab.get_engine().current_object = self.plot
                txt = mlab.text(0.01, 0.01, 'Time = 0.0',
                                width=0.35)
                self._text = txt
                self._time_changed(self.time)
        else:
            if txt is not None:
                txt.visible = False

    def _time_changed(self, value):
        txt = self._text
        if txt is not None:
            txt.text = 'Time = %.3e'%(value)

    def _extra_scalars_default(self):
        return ['vmag']


class PythonShellView(HasTraits):
    ns = Dict()
    view = View(Item('ns', editor=ShellEditor(), show_label=False))


class ViewerHandler(Handler):

    def closed(self, info, is_ok):
        """Call the viewer's on_close method when the UI is closed.
        """
        info.object.on_close()


##############################################################################
# `MayaviViewer` class.
##############################################################################
class MayaviViewer(HasTraits):
    """
    This class represents a Mayavi based viewer for the particles.  They
    are queried from a running solver.
    """

    particle_arrays = List(Instance(ParticleArrayHelper), [])
    pa_names = List(Str, [])

    interpolator = Instance(InterpolatorView)

    # The default scalar to load up when running the viewer.
    scalar = Str("rho")

    scene = Instance(MlabSceneModel, ())

    ########################################
    # Traits to pull data from a live solver.
    live_mode = Bool(False, desc='if data is obtained from a running solver '\
                                 'or from saved files')

    shell = Button('Launch Python Shell')
    host = Str('localhost', desc='machine to connect to')
    port = Int(8800, desc='port to use to connect to solver')
    authkey = Password('pysph', desc='authorization key')
    host_changed = Bool(True)
    client = Instance(MultiprocessingClient)
    controller = Property(depends_on='live_mode, host_changed')

    ########################################
    # Traits to view saved solver output.
    files = List(Str, [])
    directory = Directory()
    current_file = Str('', desc='the file being viewed currently')
    update_files = Button('Refresh')
    file_count = Range(low='_low', high='_n_files', value=0,
                       desc='the file counter')
    play = Bool(False, desc='if all files are played automatically')
    play_delay = Float(0.2, desc='the delay between loading files')
    loop = Bool(False, desc='if the animation is looped')
    # This is len(files) - 1.
    _n_files = Int(0)
    _low = Int(0)

    ########################################
    # Timer traits.
    timer = Instance(Timer)
    interval = Range(0.5, 20.0, 2.0,
                     desc='frequency in seconds with which plot is updated')

    ########################################
    # Solver info/control.
    current_time = Float(0.0, desc='the current time in the simulation')
    time_step = Float(0.0, desc='the time-step of the solver')
    iteration = Int(0, desc='the current iteration number')
    pause_solver = Bool(False, desc='if the solver should be paused')

    ########################################
    # Movie.
    record = Bool(False, desc='if PNG files are to be saved for animation')
    frame_interval = Range(1, 100, 5, desc='the interval between screenshots')
    movie_directory = Str
    # internal counters.
    _count = Int(0)
    _frame_count = Int(0)
    _last_time = Float
    _solver_data = Any
    _file_name = Str
    _particle_array_updated = Bool

    ########################################
    # The layout of the dialog created
    view = View(HSplit(
                  Group(
                    Item(name='live_mode'),
                    Group(
                        Group(
                            Item(name='directory'),
                            Item(name='current_file'),
                            Item(name='file_count'),
                            HGroup(Item(name='play'),
                                   Item(name='play_delay',
                                        label='Delay',
                                        resizable=True),
                                   Item(name='loop'),
                                   Item(name='update_files',
                                        show_label=False),
                                   padding=0,
                               ),
                            label='Saved Data',
                            selected=True,
                            enabled_when='not live_mode',
                            ),
                        Group(
                            Item(name='host'),
                            Item(name='port'),
                            Item(name='authkey'),
                            label='Connection',
                            enabled_when='live_mode',
                            ),
                        layout='tabbed'
                        ),
                    Group(
                        Group(
                              Item(name='current_time'),
                              Item(name='time_step'),
                              Item(name='iteration'),
                              Item(name='pause_solver',
                                   enabled_when='live_mode'
                                   ),
                              Item(name='interval',
                                   enabled_when='not live_mode'
                                   ),
                              label='Solver',
                             ),
                        Group(
                              Item(name='record'),
                              Item(name='frame_interval'),
                              Item(name='movie_directory'),
                              label='Movie',
                            ),
                        layout='tabbed',

                        ),
                    Group(
                          Item(name='particle_arrays',
                               style='custom',
                               show_label=False,
                               editor=ListEditor(use_notebook=True,
                                                 deletable=False,
                                                 page_name='.name'
                                                 )
                               ),
                          Item(name='interpolator',
                               style='custom',
                               show_label=False),
                          layout='tabbed'
                         ),
                    Item(name='shell', show_label=False),
                  ),
                  Group(
                    Item('scene', editor=SceneEditor(scene_class=MayaviScene),
                            height=400, width=600, show_label=False),
                  )
                ),
                resizable=True,
                title='PySPH Particle Viewer',
                height=640,
                width=1024,
                handler=ViewerHandler
                )

    ######################################################################
    # `MayaviViewer` interface.
    ######################################################################
    def on_close(self):
        self._handle_particle_array_updates()

    @on_trait_change('scene:activated')
    def start_timer(self):
        if not self.live_mode:
            # No need for the timer if we are rendering files.
            return

        # Just accessing the timer will start it.
        t = self.timer
        if not t.IsRunning():
            t.Start(int(self.interval*1000))

    @on_trait_change('scene:activated')
    def update_plot(self):

        # No need to do this if files are being used.
        if not self.live_mode:
            return

        # do not update if solver is paused
        if self.pause_solver:
            return

        if self.client is None:
            self.host_changed = True

        controller = self.controller
        if controller is None:
            return

        self.current_time = t = controller.get_t()
        self.time_step = controller.get_dt()
        self.iteration = controller.get_count()

        arrays = []
        for idx, name in enumerate(self.pa_names):
            pa = controller.get_named_particle_array(name)
            arrays.append(pa)
            pah = self.particle_arrays[idx]
            pah.set(particle_array=pa, time=t)

        self.interpolator.particle_arrays = arrays

        if self.record:
            self._do_snap()

    ######################################################################
    # Private interface.
    ######################################################################
    def _do_snap(self):
        """Generate the animation."""
        p_arrays = self.particle_arrays
        if len(p_arrays) == 0:
            return
        if self.current_time == self._last_time:
            return

        if len(self.movie_directory) == 0:
            controller = self.controller
            output_dir = controller.get_output_directory()
            movie_dir = os.path.join(output_dir, 'movie')
            self.movie_directory = movie_dir
        else:
            movie_dir = self.movie_directory
        if not os.path.exists(movie_dir):
            os.mkdir(movie_dir)

        interval = self.frame_interval
        count = self._count
        if count%interval == 0:
            fname = 'frame%06d.png'%(self._frame_count)
            p_arrays[0].scene.save_png(os.path.join(movie_dir, fname))
            self._frame_count += 1
            self._last_time = self.current_time
        self._count += 1

    @on_trait_change('host,port,authkey')
    def _mark_reconnect(self):
        if self.live_mode:
            self.host_changed = True

    @cached_property
    def _get_controller(self):
        ''' get the controller, also sets the iteration count '''
        if not self.live_mode:
            return None

        reconnect = self.host_changed
        if not reconnect:
            try:
                c = self.client.controller
            except Exception as e:
                logger.info('Error: no connection or connection closed: '\
                        'reconnecting: %s'%e)
                reconnect = True
                self.client = None
            else:
                try:
                    self.client.controller.get_count()
                except IOError:
                    self.client = None
                    reconnect = True

        if reconnect:
            self.host_changed = False
            try:
                if MultiprocessingClient.is_available((self.host, self.port)):
                    self.client = MultiprocessingClient(
                        address=(self.host, self.port),
                        authkey=self.authkey
                    )
                else:
                    logger.info(
                        'Could not connect: Multiprocessing Interface'\
                        ' not available on %s:%s'%(self.host,self.port)
                    )
                    return None
            except Exception as e:
                logger.info('Could not connect: check if solver is '\
                            'running:%s'%e)
                return None
            c = self.client.controller
            self.iteration = c.get_count()

        if self.client is None:
            return None
        else:
            return self.client.controller

    def _client_changed(self, old, new):
        if not self.live_mode:
            return

        self._clear()
        if new is None:
            return
        else:
            self.pa_names = self.client.controller.get_particle_array_names()

        self.particle_arrays = [
            self._make_particle_array_helper(self.scene, x)
            for x in self.pa_names
        ]
        self.interpolator = InterpolatorView(scene=self.scene)
        # Turn on the legend for the first particle array.
        if len(self.particle_arrays) > 0:
            self.particle_arrays[0].set(show_legend=True, show_time=True)

    def _timer_event(self):
        # catch all Exceptions else timer will stop
        try:
            self.update_plot()
        except Exception as e:
            logger.info('Exception: %s caught in timer_event'%e)

    def _interval_changed(self, value):
        t = self.timer
        if t is None:
            return
        if t.IsRunning():
            t.Stop()
            t.Start(int(value*1000))

    def _timer_default(self):
        return Timer(int(self.interval*1000), self._timer_event)

    def _pause_solver_changed(self, value):
        if self.live_mode:
            c = self.controller
            if c is None:
                return
            if value:
                c.pause_on_next()
            else:
                c.cont()

    def _record_changed(self, value):
        if value:
            self._do_snap()

    def _files_changed(self, value):
        if len(value) == 0:
            return
        else:
            d = os.path.dirname(os.path.abspath(value[0]))
            self.movie_directory = os.path.join(d, 'movie')
            self.set(directory=d, trait_change_notify=False)
        self._n_files = len(value) - 1
        self.frame_interval = 1
        fc = self.file_count
        self.file_count = 0
        if fc == 0:
            # Force an update when our original file count is 0.
            self._file_count_changed(fc)
        t = self.timer
        if not self.live_mode:
            if t.IsRunning():
                t.Stop()
        else:
            if not t.IsRunning():
                t.Stop()
                t.Start(self.interval*1000)

    def _file_count_changed(self, value):
        # Save out any updates for the previous file if needed.
        self._handle_particle_array_updates()
        # Load the new file.
        fname = self.files[value]
        self._file_name = fname
        self.current_file = os.path.basename(fname)
        # Code to read the file, create particle array and setup the helper.
        data = load(fname)
        solver_data = data["solver_data"]
        arrays = data["arrays"]
        self._solver_data = solver_data
        self.current_time = t = float(solver_data['t'])
        self.time_step = float(solver_data['dt'])
        self.iteration = int(solver_data['count'])
        names = list(arrays.keys())
        pa_names = self.pa_names

        if len(pa_names) == 0:
            self.interpolator = InterpolatorView(scene=self.scene)
            self.pa_names = names
            pas = []
            for name in names:
                pa = arrays[name]
                pah = self._make_particle_array_helper(self.scene, name)
                # Must set this after setting the scene.
                pah.set(particle_array=pa, time=t)
                pas.append(pah)
            self.particle_arrays = pas
        else:
            for idx, name in enumerate(pa_names):
                pa = arrays[name]
                pah = self.particle_arrays[idx]
                pah.set(particle_array=pa, time=t)

        self.interpolator.particle_arrays = list(arrays.values())

        if self.record:
            self._do_snap()

    def _loop_changed(self, value):
        if value and self.play:
            self._play_changed(self.play)

    def _play_changed(self, value):
        t = self.timer
        if value:
            t.Stop()
            t.callable = self._play_event
            t.Start(1000*self.play_delay)
        else:
            t.Stop()
            t.callable = self._timer_event

    def _clear(self):
        self.pa_names = []
        self.scene.mayavi_scene.children[:] = []

    def _play_event(self):
        nf = self._n_files
        pc = self.file_count
        pc += 1
        if pc > nf:
            if self.loop:
                pc = 0
            else:
                self.timer.Stop()
                pc = nf
        self.file_count = pc
        self._handle_particle_array_updates()

    def _play_delay_changed(self):
        if self.play:
            self._play_changed(self.play)

    def _scalar_changed(self, value):
        for pa in self.particle_arrays:
            pa.scalar = value

    def _update_files_fired(self):
        fc = self.file_count
        files = glob_files(self.files[fc])
        sort_file_list(files)
        self.files = files
        self.file_count = fc
        if self.play:
            self._play_changed(self.play)

    def _shell_fired(self):
        ns = dict(viewer=self, particle_arrays=self.particle_arrays,
                  interpolator=self.interpolator, scene=self.scene,
                  mlab=self.scene.mlab)
        obj = PythonShellView(ns=ns)
        obj.edit_traits()

    def _directory_changed(self, d):
        ext = os.path.splitext(self.files[-1])[1]
        files = glob.glob(os.path.join(d, '*' + ext))
        if len(files) > 0:
            self._clear()
            sort_file_list(files)
            self.files = files
            self.file_count = min(self.file_count, len(files))
        else:
            pass

    def _live_mode_changed(self, value):
        if value:
            self._file_name = ''
            self.client = None
            self._clear()
            self._mark_reconnect()
            self.start_timer()
        else:
            self.client = None
            self._clear()
            self.timer.Stop()

    def _particle_array_helper_updated(self, value):
        self._particle_array_updated = True

    def _handle_particle_array_updates(self):
        # Called when the particle array helper fires an updated event.
        if self._particle_array_updated and self._file_name:
            sd = self._solver_data
            arrays = [x.particle_array for x in self.particle_arrays]
            dump(self._file_name, arrays, sd)
            self._particle_array_updated = False

    def _make_particle_array_helper(self, scene, name):
        pah = ParticleArrayHelper(scene=scene, name=name, scalar=self.scalar)
        pah.on_trait_change(self._particle_array_helper_updated, 'updated')
        return pah


######################################################################
def usage():
    print("""Usage:
pysph view [-v] <trait1=value> <trait2=value> [directory or files.npz]

If a directory or *.npz files are not supplied it will connect to a running
solver, if not it will display the given files.

The arguments <trait1=value> are optional settings like host, port and authkey
etc.  The following traits are available:

  scalar        -- the default scalar to display on the view.

  host          -- hostname/IP address to connect to.
  port          -- Port to connect to
  authkey       -- authorization key to use.
  interval      -- time interval to refresh display
  pause_solver  -- Set True/False, will pause running solver

  movie_directory -- directory to dump movie files (automatically set if not
                       supplied)
  record        -- True/False: record movie, i.e. store screenshots of display.

  play          -- True/False: Play all stored data files.
  loop          -- True/False: Loop over data files.

Options:
--------

  -h/--help   prints this message.

  -v          sets verbose mode which will print solver connection
              status failures on stdout.

Examples::
----------

  $ pysph view scalar=u play=True loop=True elliptical_drop_output/
  $ pysph view ellptical_drop_100.npz
  $ pysph view interval=10 host=localhost port=8900

""")

def error(msg):
    print(msg)
    sys.exit()

def main(args=None):
    if args is None:
        args = sys.argv[1:]

    if '-h' in args or '--help' in args:
        usage()
        sys.exit(0)

    if '-v' in args:
        logger.addHandler(logging.StreamHandler())
        logger.setLevel(logging.INFO)
        args.remove('-v')

    kw = {}
    files = []
    for arg in args:
        if '=' not in arg:
            if arg.endswith(output_formats):
                files.extend(glob.glob(arg))
                continue
            elif os.path.isdir(arg):
                _files = glob.glob(os.path.join(arg, '*.hdf5'))
                if len(_files) == 0:
                    _files = glob.glob(os.path.join(arg, '*.npz'))
                files.extend(_files)
                continue
            else:
                usage()
                sys.exit(1)
        key, arg = [x.strip() for x in arg.split('=')]
        try:
            val = eval(arg, math.__dict__)
            # this will fail if arg is a string.
        except NameError:
            val = arg
        kw[key] = val

    sort_file_list(files)
    live_mode = len(files) == 0

    # If we set the particle arrays before the scene is activated, the arrays
    # are not displayed on screen so we use do_later to set the  files.
    m = MayaviViewer(live_mode=live_mode)
    do_later(m.set, files=files, **kw)
    m.configure_traits()

if __name__ == '__main__':
    main()



#Author: Anshuman Kumar

try:
    # This is for Python-2.6.x
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase

try:
    from unittest import mock
except ImportError:
    import mock

from pysph.solver.application import Application
from pysph.solver.solver import Solver

class MockApp(Application):
    def add_user_options(self,group):
        group.add_argument( "--testarg", action="store", type=float,
                dest="testarg", default=int(10.0), help="Test Argument")
        
    def consume_user_options(self):
        self.testarg  = self.options.testarg

    def create_particles(self):
        return []

    def create_equations(self):
        return []

    def create_solver(self):
        solver = Solver()
        solver.particles =[]
        solver.solve = mock.Mock()
        solver.setup = mock.Mock()
        return solver

    def create_nnps(self):
        nnps = mock.Mock()
        return nnps
 
class TestApplication(TestCase):

    # Test When testarg is  notpassed
    def test_user_options_false(self):
        #Given
        app = MockApp()

        #When
        args = []
        app.run(args)
        record = app.testarg

        #Then
        expected = 10.0
        error_message = "Expected %f, got %f"%(expected, app.testarg)
        self.assertEqual(expected,app.testarg,error_message)

    # Test When testarg is passed
    def test_user_options_true(self):
        #Given
        app = MockApp()

        #When
        args = ['--testarg', '20']
        app.run(args)

        #Then
        expected = 20.0
        error_message = "Expected %f, got %f"%(expected, app.testarg)
        self.assertEqual(expected,app.testarg,error_message)




# Standard library imports.
try:
    # This is for Python-2.6.x
    import unittest2 as unittest
except ImportError:
    import unittest

# Library imports.
import numpy as np

# Local imports.
from pysph.base.utils import get_particle_array
from pysph.sph.equation import Equation, Group
from pysph.sph.acceleration_eval import (AccelerationEval,
    check_equation_array_properties)
from pysph.sph.basic_equations import SummationDensity
from pysph.base.kernels import CubicSpline
from pysph.base.nnps import LinkedListNNPS as NNPS
from pysph.sph.sph_compiler import SPHCompiler

from pysph.base.reduce_array import serial_reduce_array


class TestEquation(Equation):
    def initialize(self, d_idx, d_rho, d_V):
        d_rho[d_idx] = d_V[d_idx]

    def loop(self, d_idx, d_rho, s_idx, s_m, s_u, WIJ):
        d_rho[d_idx] += s_m[s_idx]*WIJ

    def post_loop(self, d_idx, d_rho, s_idx, s_m, s_V, WIJ):
        d_rho[d_idx] += s_m[s_idx]*WIJ


class FindTotalMass(Equation):
    def initialize(self, d_idx, d_m, d_total_mass):
        # FIXME: This is stupid and should be fixed if we add a separate
        # initialize_once function or so.
        d_total_mass[0] = 0.0

    def post_loop(self, d_idx, d_m, d_total_mass):
        d_total_mass[0] += d_m[d_idx]


class TestCheckEquationArrayProps(unittest.TestCase):

    def test_should_pass_when_properties_exist(self):
        # Given
        f = get_particle_array(name='f')

        # When
        eq = SummationDensity(dest='f', sources=['f'])

        # Then
        check_equation_array_properties(eq, [f])

    def test_should_fail_when_props_dont_exist(self):
        # Given
        f = get_particle_array(name='f')

        # When
        eq = TestEquation(dest='f', sources=['f'])

        # Then
        self.assertRaises(RuntimeError,
                          check_equation_array_properties, eq, [f])

    def test_should_fail_when_src_props_dont_exist(self):
        # Given
        f = get_particle_array(name='f')
        f.add_property('V')
        s = get_particle_array(name='s')

        # When
        eq = TestEquation(dest='f', sources=['f', 's'])

        # Then
        self.assertRaises(RuntimeError,
                          check_equation_array_properties, eq, [f, s])

    def test_should_pass_when_src_props_exist(self):
        # Given
        f = get_particle_array(name='f')
        f.add_property('V')
        s = get_particle_array(name='s')
        s.add_property('V')

        # When
        eq = TestEquation(dest='f', sources=['f', 's'])

        # Then
        check_equation_array_properties(eq, [f, s])

    def test_should_check_constants(self):
        # Given
        f = get_particle_array(name='f')

        # When
        eq = FindTotalMass(dest='f', sources=['f'])

        # Then.
        self.assertRaises(RuntimeError,
                          check_equation_array_properties, eq, [f])

        # When.
        f.add_constant('total_mass', 0.0)

        # Then.
        check_equation_array_properties(eq, [f])


class SimpleEquation(Equation):
    def __init__(self, dest, sources):
        super(SimpleEquation, self).__init__(dest, sources)
        self.count = 0

    def initialize(self, d_idx, d_u, d_au):
        d_u[d_idx] = 0.0
        d_au[d_idx] = 0.0

    def loop(self, d_idx, d_au, s_idx, s_m):
        #print d_idx, s_idx
        d_au[d_idx] += s_m[s_idx]

    def post_loop(self, d_idx, d_u, d_au):
        d_u[d_idx] = d_au[d_idx]

    def converged(self):
        self.count += 1
        result = self.count - 1
        if result > 0:
            # Reset the count for the next loop.
            self.count = 0
        return result

class MixedTypeEquation(Equation):
    def initialize(self, d_idx, d_u, d_au, d_pid, d_tag):
        d_u[d_idx] = 0.0 + d_pid[d_idx]
        d_au[d_idx] = 0.0 + d_tag[d_idx]

    def loop(self, d_idx, d_au, s_idx, s_m, s_pid, s_tag):
        #print d_idx, s_idx
        d_au[d_idx] += s_m[s_idx] + s_pid[s_idx] + s_tag[s_idx]

    def post_loop(self, d_idx, d_u, d_au, d_pid):
        d_u[d_idx] = d_au[d_idx] + d_pid[d_idx]


class SimpleReduction(Equation):
    def reduce(self, dst):
        dst.total_mass[0] = serial_reduce_array(dst.array.m, op='sum')


class TestAccelerationEval1D(unittest.TestCase):
    def setUp(self):
        self.dim = 1
        n = 10
        dx = 1.0/(n-1)
        x = np.linspace(0, 1, n)
        m = np.ones_like(x)
        h = np.ones_like(x)*dx*1.05
        pa = get_particle_array(name='fluid', x=x, h=h, m=m)
        self.pa = pa

    def _make_accel_eval(self, equations, cache_nnps=False):
        arrays = [self.pa]
        kernel = CubicSpline(dim=self.dim)
        a_eval = AccelerationEval(
            particle_arrays=arrays, equations=equations, kernel=kernel
        )
        comp = SPHCompiler(a_eval, integrator=None)
        comp.compile()
        nnps = NNPS(dim=kernel.dim, particles=arrays, cache=cache_nnps)
        nnps.update()
        a_eval.set_nnps(nnps)
        return a_eval

    def test_should_support_constants(self):
        # Given
        pa = self.pa
        pa.add_constant('total_mass', 0.0)
        equations = [FindTotalMass (dest='fluid', sources=['fluid'])]
        a_eval = self._make_accel_eval(equations)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        self.assertEqual(pa.total_mass, 10.0)

    def test_should_not_iterate_normal_group(self):
        # Given
        pa = self.pa
        equations = [SimpleEquation(dest='fluid', sources=['fluid'])]
        a_eval = self._make_accel_eval(equations)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        expect = np.asarray([3., 4., 5., 5., 5., 5., 5., 5.,  4.,  3.])
        self.assertListEqual(list(pa.u), list(expect))

    def test_should_work_with_cached_nnps(self):
        # Given
        pa = self.pa
        equations = [SimpleEquation(dest='fluid', sources=['fluid'])]
        a_eval = self._make_accel_eval(equations, cache_nnps=True)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        expect = np.asarray([3., 4., 5., 5., 5., 5., 5., 5.,  4.,  3.])
        self.assertListEqual(list(pa.u), list(expect))

    def test_should_iterate_iterated_group(self):
        # Given
        pa = self.pa
        equations = [Group(
            equations=[SimpleEquation(dest='fluid', sources=['fluid']),
                       SimpleEquation(dest='fluid', sources=['fluid']),
                      ],
            iterate=True
        )]
        a_eval = self._make_accel_eval(equations)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        expect = np.asarray([3., 4., 5., 5., 5., 5., 5., 5.,  4.,  3.])*2
        self.assertListEqual(list(pa.u), list(expect))

    def test_should_iterate_nested_groups(self):
        pa = self.pa
        equations = [Group(
            equations=[
                Group(
                    equations=[SimpleEquation(dest='fluid', sources=['fluid'])]
                ),
                Group(
                    equations=[SimpleEquation(dest='fluid', sources=['fluid'])],
                ),
            ],
            iterate=True,
        )]
        a_eval = self._make_accel_eval(equations)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        expect = np.asarray([3., 4., 5., 5., 5., 5., 5., 5.,  4.,  3.])
        self.assertListEqual(list(pa.u), list(expect))

    def test_should_run_reduce(self):
        # Given.
        pa = self.pa
        pa.add_constant('total_mass', 0.0)
        equations = [SimpleReduction(dest='fluid', sources=['fluid'])]
        a_eval = self._make_accel_eval(equations)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        expect = np.sum(pa.m)
        self.assertAlmostEqual(pa.total_mass[0], expect, 14)

    def test_should_work_with_non_double_arrays(self):
        # Given
        pa = self.pa
        equations = [MixedTypeEquation(dest='fluid', sources=['fluid'])]
        a_eval = self._make_accel_eval(equations)

        # When
        a_eval.compute(0.1, 0.1)

        # Then
        expect = np.asarray([3., 4., 5., 5., 5., 5., 5., 5.,  4.,  3.])
        self.assertListEqual(list(pa.u), list(expect))



"""Test if dumping and loading files works in parallel correctly.
"""

import mpi4py.MPI as mpi
import numpy as np
from os.path import join
import shutil
from tempfile import mkdtemp

from pysph.base.particle_array import ParticleArray
from pysph.solver.utils import dump, load

def assert_lists_same(l1, l2):
    expect = list(sorted(l1))
    result = list(sorted(l2))
    assert l1 == l2, "Expected %s, got %s"%(l1, l2)

comm = mpi.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

root = mkdtemp()
filename = join(root, 'test.npz')

x = np.ones(5, dtype=float)*rank
pa = ParticleArray(name='fluid', constants={'c1': 0.0, 'c2': [0.0, 0.0]}, x=x)

try:
    dump(filename, [pa], {}, mpi_comm=comm)
    if rank == 0:
        data = load(filename)
        pa1 = data["arrays"]["fluid"]

        assert_lists_same(pa.properties.keys(), pa1.properties.keys())
        assert_lists_same(pa.constants.keys(), pa1.constants.keys())

        expect = np.ones(5*size)
        for i in range(size):
            expect[5*i:5*(i+1)] = i

        assert np.allclose(pa1.x, expect, atol=1e-14), \
            "Expected %s, got %s"%(expect, pa1.x)
finally:
    shutil.rmtree(root)



"""Example for the Noh's cylindrical implosion test. (15 mins)
"""

# NumPy and standard library imports
import numpy

# PySPH base and carray imports
from pysph.base.utils import get_particle_array as gpa
from pysph.solver.application import Application
from pysph.sph.scheme import GasDScheme


# problem constants
dim = 2
gamma = 5.0/3.0
gamma1 = gamma - 1.0

# scheme constants
alpha1 = 1.0
alpha2 = 0.1
beta = 2.0
kernel_factor = 1.2

# numerical constants
dt = 1e-4
tf = 0.4

# domain and particle spacings
xmin = ymin = -0.5
xmax = ymax = 0.5

nx = ny = 50
dx = (xmax-xmin)/nx
dxb2 = 0.5 * dx

# initial values
h0 = kernel_factor*dx
rho0 = 1.0
e0 = 1e-6
m0 = dx*dx * rho0
vr = -1.0


class NohImplosion(Application):
    def create_particles(self):
        x, y = numpy.mgrid[
            xmin:xmax:dx, ymin:ymax:dx]

        # positions
        x = x.ravel(); y = y.ravel()

        rho = numpy.ones_like(x) * rho0
        m = numpy.ones_like(x) * m0
        e = numpy.ones_like(x) * e0
        h = numpy.ones_like(x) * h0
        p = gamma1*rho*e

        u = numpy.ones_like(x)
        v = numpy.ones_like(x)

        sin, cos, arctan = numpy.sin, numpy.cos, numpy.arctan2
        for i in range(x.size):
            theta = arctan(y[i],x[i])
            u[i] = vr*cos(theta)
            v[i] = vr*sin(theta)

        fluid = gpa(name='fluid', x=x,y=y,m=m,rho=rho, h=h,u=u,v=v,p=p,e=e)
        self.scheme.setup_properties([fluid])

        print("Noh's problem with %d particles"%(fluid.get_number_of_particles()))

        return [fluid,]

    def create_scheme(self):
        s = GasDScheme(
            fluids=['fluid'], solids=[], dim=dim, gamma=gamma,
            kernel_factor=kernel_factor, alpha1=alpha1, alpha2=alpha2,
            beta=beta, adaptive_h_scheme="gsph",
            update_alpha1=True, update_alpha2=True
        )
        s.configure_solver(dt=dt, tf=tf, adaptive_timestep=False)
        return s


if __name__ == '__main__':
    app = NohImplosion()
    app.run()



"""Test the ParticleArrayExchange object with the following data

25 particles are created on a rectangular grid with the following
processor assignment:

2---- 3---- 3---- 3---- 3
|     |     |     |     |
1---- 1---- 1---- 2---- 2
|     |     |     |     |
0---- 0---- 1---- 1---- 1
|     |     |     |     |
0---- 0---- 0---- 0---- 0
|     |     |     |     |
0---- 0---- 0---- 0---- 0


We assume that after a load balancing step, we have the following
distribution:

1---- 1---- 1---- 3---- 3
|     |     |     |     |
1---- 1---- 1---- 3---- 3
|     |     |     |     |
0---- 0---- 0---- 3---- 3
|     |     |     |     |
0---- 0---- 2---- 2---- 2
|     |     |     |     |
0---- 0---- 2---- 2---- 2

We create a ParticleArray to represent the initial distribution and
use ParticleArrayExchange to move the data by manually setting the
particle import/export lists. We require that the test be run with 4
processors.

"""
import mpi4py.MPI as mpi
import numpy as np

from pysph.parallel.parallel_manager import ParticleArrayExchange
from pysph.base.utils import get_particle_array_wcsph

comm = mpi.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

if size != 4:
    if rank == 0:
        raise RuntimeError("Run this test with 4 processors")

# create the initial distribution
if rank == 0:
    numPoints = 12
    x = np.array( [0.0, 1.0, 2.0, 3.0, 4.0,
                   0.0, 1.0, 2.0, 3.0, 4.0,
                   0.0, 1.0] )

    y = np.array( [0.0, 0.0, 0.0, 0.0, 0.0,
                   1.0, 1.0, 1.0, 1.0, 1.0,
                   2.0, 2.0] )

    gid = np.array( [0, 1, 2, 3, 4, 5,
                     6, 7, 8, 9, 10,
                     11], dtype=np.uint32 )

    exportLocalids = np.array( [2, 3, 4, 7, 8, 9], dtype=np.uint32 )
    exportProcs = np.array( [2, 2, 2, 2, 2, 2], dtype=np.int32 )
    numExport = 6

    parts = np.ones(shape=numPoints, dtype=np.int32) * rank

if rank == 1:
    numPoints = 6
    x = np.array( [2.0, 3.0, 4.0,
                   0.0, 1.0, 2.0] )

    y = np.array( [2.0, 2.0, 2.0,
                   3.0, 3.0, 3.0] )

    gid = np.array( [12, 13, 14, 15,
                     16, 17], dtype=np.uint32 )

    exportLocalids = np.array( [0, 1, 2], dtype=np.uint32 )
    exportProcs = np.array( [0, 3, 3], dtype=np.int32 )
    numExport = 3

    parts = np.ones(shape=numPoints, dtype=np.int32) * rank

if rank == 2:
    numPoints = 3
    x = np.array( [4.0, 5.0, 0.0] )
    y = np.array( [3.0, 3.0, 4.0] )
    gid = np.array( [18, 19, 20], dtype=np.uint32 )

    exportLocalids = np.array( [0, 1, 2], dtype=np.uint32 )
    exportProcs = np.array( [3, 3, 1], dtype=np.int32 )
    numExport = 3

    parts = np.ones(shape=numPoints, dtype=np.int32) * rank

if rank == 3:
    numPoints = 4
    x = np.array( [1.0, 2.0, 3.0, 4.0] )
    y = np.array( [4.0, 4.0, 4.0, 4.0] )
    gid = np.array( [21, 22, 23, 24], dtype=np.uint32 )

    exportLocalids = np.array( [0,1], dtype=np.uint32 )
    exportProcs = np.array( [1, 1], dtype=np.int32 )
    numExport = 2

    parts = np.ones(shape=numPoints, dtype=np.int32) * rank

# Gather the Global data on root
X = np.zeros(shape=25, dtype=np.float64)
Y = np.zeros(shape=25, dtype=np.float64)
GID = np.zeros(shape=25, dtype=np.uint32)

displacements = np.array( [12, 6, 3, 4], dtype=np.int32 )

comm.Gatherv(sendbuf=[x, mpi.DOUBLE], recvbuf=[X, (displacements, None)], root=0)
comm.Gatherv(sendbuf=[y, mpi.DOUBLE], recvbuf=[Y, (displacements, None)], root=0)
comm.Gatherv(sendbuf=[gid, mpi.UNSIGNED_INT], recvbuf=[GID, (displacements, None)], root=0)

# broadcast global X, Y and GID to everyone
comm.Bcast(buf=X, root=0)
comm.Bcast(buf=Y, root=0)
comm.Bcast(buf=GID, root=0)

# create the local particle arrays and exchange objects
pa = get_particle_array_wcsph(name='test', x=x, y=y, gid=gid)
pae = ParticleArrayExchange(pa_index=0, pa=pa, comm=comm)

# set the export indices for each array
pae.reset_lists()
pae.numParticleExport = numExport
pae.exportParticleLocalids.resize(numExport)
pae.exportParticleLocalids.set_data( exportLocalids )

pae.exportParticleProcs.resize( numExport )
pae.exportParticleProcs.set_data( exportProcs )

# call lb_balance with these lists
pae.lb_exchange_data()

# All arrays must be local after lb_exchange_data
assert( pa.num_real_particles == pa.get_number_of_particles() )
assert( np.allclose(pa.tag, 0) )

# now check the data on each array
numParticles = 6
if rank == 0:
    numParticles = 7

assert( pa.num_real_particles == numParticles )

for i in range(numParticles):
    assert( abs(X[pa.gid[i]] - pa.x[i]) < 1e-15 )
    assert( abs(Y[pa.gid[i]] - pa.y[i]) < 1e-15 )
    assert( GID[pa.gid[i]] == pa.gid[i] )



"""Test code for Cython code generation.
"""
import unittest
from textwrap import dedent
from math import pi, sin

from pysph.base.config import get_config, set_config
from pysph.base.cython_generator import (CythonGenerator, CythonClassHelper,
    KnownType, all_numeric)

def declare(*args):
    pass

class BasicEq:
    def __init__(self, hidden=None, rho=0.0, c=0.0):
        self.rho = rho
        self.c = c
        self._hidden = ['a', 'b']

class EqWithMethod(BasicEq):
    def func(self, d_idx=0, d_x=[0.0, 0.0]):
        tmp = abs(self.rho*self.c)*sin(pi*self.c)
        d_x[d_idx] = d_x[d_idx]*tmp

class EqWithReturn(BasicEq):
    def func(self, d_idx=0, d_x=[0.0, 0.0]):
        return d_x[d_idx]

class EqWithKnownTypes:
    def some_func(self, d_idx, d_p, WIJ, DWIJ, user, d_user, s_user):
        d_p[d_idx] = WIJ*DWIJ[0]

class EqWithMatrix:
    def func(self, d_idx, d_x=[0.0, 0.0]):
        mat = declare('matrix((2,2))')
        mat[0][0] = d_x[d_idx]
        vec = declare('matrix((3,))')
        vec[0] = d_x[d_idx]

class EqWithDeclare:
    def func(self, d_idx, d_x=[0.0, 0.0]):
        val = declare('float')
        val = d_x[d_idx]
        index = declare('unsigned int')
        index = d_idx


def func_with_return(d_idx, d_x, x=0.0):
    x += 1
    return d_x[d_idx] + x

def simple_func(d_idx, d_x, x=0.0):
    d_x[d_idx] += x


class TestBase(unittest.TestCase):
    def assert_code_equal(self, result, expect):
        expect = expect.strip()
        result = result.strip()
        msg = 'EXPECTED:\n%s\nGOT:\n%s'%(expect, result)
        self.assertEqual(expect, result, msg)


class TestMiscUtils(TestBase):

    def test_all_numeric(self):
        x = [1, 2, 3.0]
        self.assertTrue(all_numeric(x))
        try:
            x = [0.0, 1, long(3)]
        except NameError:
            x = [0.0, 1, 3]
        self.assertTrue(all_numeric(x))
        x = [0.0, 1.0, '']
        self.assertFalse(all_numeric(x))

    def test_detect_type(self):
        cases = [(('d_something', None), 'double*'),
                 (('s_something', None), 'double*'),
                 (('d_idx', 0), 'long'),
                 (('x', 1), 'long'),
                 (('s', 'asdas'), 'str'),
                 (('junk', 1.0), 'double'),
                 (('y', [0.0, 1]), 'double*'),
                 (('y', [0, 1, 0]), 'double*'),
                 (('y', None), 'object'),
                ]
        cg = CythonGenerator()
        for args, expect in cases:
            msg = 'detect_type(*%r) != %r'%(args, expect)
            self.assertEqual(cg.detect_type(*args), expect, msg)


    def test_cython_class_helper(self):
        code = ('def f(self, x):',
                '        x += 1\n        return x+1')
        c = CythonClassHelper(name='A', public_vars={'x': 'double'},
                              methods=[code])
        expect = dedent("""
        cdef class A:
            cdef public double x
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            def f(self, x):
                x += 1
                return x+1
        """)
        self.assert_code_equal(c.generate().strip(), expect.strip())


class TestCythonCodeGenerator(TestBase):
    def setUp(self):
        get_config().use_openmp = False

    def tearDown(self):
        set_config(None)

    def test_simple_constructor(self):
        cg = CythonGenerator()
        cg.parse(BasicEq())
        expect = dedent("""
        cdef class BasicEq:
            cdef public list _hidden
            cdef public double c
            cdef public double rho
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_simple_method(self):
        cg = CythonGenerator()
        cg.parse(EqWithMethod())
        expect = dedent("""
        cdef class EqWithMethod:
            cdef public list _hidden
            cdef public double c
            cdef public double rho
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline void func(self, long d_idx, double* d_x):
                cdef double tmp
                tmp = abs(self.rho*self.c)*sin(pi*self.c)
                d_x[d_idx] = d_x[d_idx]*tmp
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_honors_use_openmp_setting(self):
        # When
        get_config().use_openmp = True
        # Then
        cg = CythonGenerator()
        cg.parse(EqWithMethod())
        expect = dedent("""
        cdef class EqWithMethod:
            cdef public list _hidden
            cdef public double c
            cdef public double rho
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline void func(self, long d_idx, double* d_x) nogil:
                cdef double tmp
                tmp = abs(self.rho*self.c)*sin(pi*self.c)
                d_x[d_idx] = d_x[d_idx]*tmp
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_python_methods(self):
        cg = CythonGenerator(python_methods=True)
        cg.parse(EqWithMethod())
        expect = dedent("""
        cdef class EqWithMethod:
            cdef public list _hidden
            cdef public double c
            cdef public double rho
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline void func(self, long d_idx, double* d_x):
                cdef double tmp
                tmp = abs(self.rho*self.c)*sin(pi*self.c)
                d_x[d_idx] = d_x[d_idx]*tmp

            cpdef py_func(self, long d_idx, double[:] d_x):
                self.func(d_idx, &d_x[0])
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

        cg.parse(EqWithReturn())
        expect = dedent("""
        cdef class EqWithReturn:
            cdef public list _hidden
            cdef public double c
            cdef public double rho
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline double func(self, long d_idx, double* d_x):
                return d_x[d_idx]

            cpdef double py_func(self, long d_idx, double[:] d_x):
                return self.func(d_idx, &d_x[0])
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

        cg.parse(func_with_return)
        expect = dedent("""
        cdef inline double func_with_return(long d_idx, double* d_x, double x):
            x += 1
            return d_x[d_idx] + x

        cpdef double py_func_with_return(long d_idx, double[:] d_x, double x):
            return func_with_return(d_idx, &d_x[0], x)
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())


    def test_method_with_return(self):
        cg = CythonGenerator()
        cg.parse(EqWithReturn())
        expect = dedent("""
        cdef class EqWithReturn:
            cdef public list _hidden
            cdef public double c
            cdef public double rho
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline double func(self, long d_idx, double* d_x):
                return d_x[d_idx]
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_method_with_matrix(self):
        cg = CythonGenerator()
        cg.parse(EqWithMatrix())
        expect = dedent("""
        cdef class EqWithMatrix:
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline void func(self, long d_idx, double* d_x):
                cdef double mat[2][2]
                mat[0][0] = d_x[d_idx]
                cdef double vec[3]
                vec[0] = d_x[d_idx]
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_method_with_declare(self):
        cg = CythonGenerator()
        cg.parse(EqWithDeclare())
        expect = dedent("""
        cdef class EqWithDeclare:
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline void func(self, long d_idx, double* d_x):
                cdef float val
                val = d_x[d_idx]
                cdef unsigned int index
                index = d_idx
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_method_with_known_types(self):
        cg = CythonGenerator(
            known_types={'WIJ':0.0, 'DWIJ':[0.0, 0.0, 0.0],
                         'user': KnownType('ndarray'),
                         'd_user': KnownType('long*'),
                         's_user': KnownType('int*'),
                         }
        )
        cg.parse(EqWithKnownTypes())
        expect = dedent("""
        cdef class EqWithKnownTypes:
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)

            cdef inline void some_func(self, long d_idx, double* d_p, double WIJ, double* DWIJ, ndarray user, long* d_user, int* s_user):
                d_p[d_idx] = WIJ*DWIJ[0]
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

    def test_wrap_function(self):
        cg = CythonGenerator()
        cg.parse(func_with_return)
        expect = dedent("""
        cdef inline double func_with_return(long d_idx, double* d_x, double x):
            x += 1
            return d_x[d_idx] + x
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())

        cg.parse(simple_func)
        expect = dedent("""
        cdef inline void simple_func(long d_idx, double* d_x, double x):
            d_x[d_idx] += x
        """)
        self.assert_code_equal(cg.get_code().strip(), expect.strip())


if __name__ == '__main__':
    unittest.main()



"""
Module contains some common functions.
"""

# standard imports
import pickle
import numpy
import sys
import os

from pysph.base.particle_array import ParticleArray
from pysph.base.utils import get_particle_array, get_particles_info
from pysph.solver.output import load, dump, output_formats

HAS_PBAR = True
try:
    import progressbar
except ImportError:
    HAS_PBAR = False

import pysph

def check_array(x, y):
    """Check if two arrays are equal with an absolute tolerance of
    1e-16."""
    return numpy.allclose(x, y, atol=1e-16, rtol=0)

def get_distributed_particles(pa, comm, cell_size):
    # FIXME: this can be removed once the examples all use Application.
    from pysph.parallel.load_balancer import LoadBalancer
    rank = comm.Get_rank()
    num_procs = comm.Get_size()

    if rank == 0:
        lb = LoadBalancer.distribute_particles(pa, num_procs=num_procs,
                                               block_size=cell_size)
    else:
        lb = None

    particles = comm.scatter(lb, root=0)

    return particles

def get_array_by_name(arrays, name):
    """Given a list of arrays and the name of the desired array, return the
    desired array.
    """
    for array in arrays:
        if array.name == name:
            return array

################################################################################
# `PBar` class.
###############################################################################
class PBar(object):
    """A simple wrapper around the progressbar so it works if a user has
    it installed or not.
    """
    def __init__(self, maxval, show=True):
        bar = None
        self.count = 0
        self.maxval = maxval
        self.show = show
        if HAS_PBAR and show:
            widgets = [progressbar.Percentage(), ' ', progressbar.Bar(),
                       progressbar.ETA()]
            bar = progressbar.ProgressBar(widgets=widgets,
                                          maxval=maxval).start()
        self.bar = bar

    def update(self, delta=1):
        self.count += delta
        if self.bar is not None:
            self.bar.update(self.count)
        elif self.show:
            sys.stderr.write('\r%d%%'%int(self.count*100/self.maxval))
            sys.stderr.flush()

    def finish(self):
        if self.bar is not None:
            self.bar.finish()
        elif self.show:
            sys.stderr.write('\r100%\n')

            sys.stderr.flush()


class FloatPBar(object):
    def __init__(self, t_initial, t_final, show=True):
        self.ticks = 1000
        self.bar = PBar(self.ticks, show)
        self.t_initial = t_initial
        self.t_final = t_final
        self.t_diff = t_final - t_initial

    def update(self, time):
        expected_count = int((time - self.t_initial)/self.t_diff*self.ticks)
        expected_count = min(expected_count, self.ticks)
        diff = max(expected_count - self.bar.count, 0)
        if diff > 0:
            self.bar.update(diff)

    def finish(self):
        self.bar.finish()

##############################################################################
# friendly mkdir  from http://code.activestate.com/recipes/82465/.
##############################################################################
def mkdir(newdir):
    """works the way a good mkdir should :)
        - already exists, silently complete
        - regular file in the way, raise an exception
        - parent directory(ies) does not exist, make them as well
    """
    if os.path.isdir(newdir):
        pass

    elif os.path.isfile(newdir):
        raise OSError("a file with the same name as the desired " \
                      "dir, '%s', already exists." % newdir)

    else:
        head, tail = os.path.split(newdir)

        if head and not os.path.isdir(head):
            mkdir(head)

        if tail:
            try:
                os.mkdir(newdir)
            # To prevent race in mpi runs
            except OSError as e:
                import errno
                if e.errno == errno.EEXIST and os.path.isdir(newdir):
                    pass
                else:
                    raise


def get_pysph_root():
    return os.path.split(pysph.__file__)[0]

def dump_v1(filename, particles, solver_data, detailed_output=False,
         only_real=True, mpi_comm=None):
    """Dump the given particles and solver data to the given filename using
    version 1.  This is mainly used only for testing that we can continue
    to load older versions of the data files.
    """

    all_array_data = {}
    output_data = {"arrays":all_array_data, "solver_data":solver_data}

    for array in particles:
        all_array_data[array.name] = array.get_property_arrays(
            all=detailed_output, only_real=only_real)

    # Gather particle data on root
    if mpi_comm is not None:
        all_array_data = _gather_array_data(all_array_data, mpi_comm)

    output_data['arrays'] = all_array_data

    if mpi_comm is None or mpi_comm.Get_rank() == 0:
        numpy.savez(filename, version=1, **output_data)


def load_and_concatenate(prefix,nprocs=1,directory=".",count=None):
    """Load the results from multiple files.

    Given a filename prefix and the number of processors, return a
    concatenated version of the dictionary returned via load.

    Parameters
    ----------

    prefix : str
        A filename prefix for the output file.

    nprocs : int
        The number of processors (files) to read

    directory : str
        The directory for the files

    count : int
        The file iteration count to read. If None, the last available
        one is read

    """

    if count is None:
        counts = [i.rsplit('_',1)[1][:-4] for i in os.listdir(directory) if i.startswith(prefix) and i.endswith('.npz')]
        counts = sorted( [int(i) for i in counts] )
        count = counts[-1]

    arrays_by_rank = {}

    for rank in range(nprocs):
        fname = os.path.join(directory, prefix+'_'+str(rank)+'_'+str(count)+'.npz')

        data = load(fname)
        arrays_by_rank[rank] = data["arrays"]

    arrays = _concatenate_arrays(arrays_by_rank, nprocs)

    data["arrays"] = arrays

    return data

def _concatenate_arrays(arrays_by_rank, nprocs):
    """Concatenate arrays into one single particle array. """

    if nprocs <= 0:
        return 0

    array_names = arrays_by_rank[0].keys()
    first_processors_arrays = arrays_by_rank[0]

    if nprocs > 1:
        ret = {}
        for array_name in array_names:
            first_array = first_processors_arrays[array_name]
            for rank in range(1,nprocs):
                other_processors_arrays = arrays_by_rank[rank]
                other_array = other_processors_arrays[array_name]

                # append the other array to the first array
                first_array.append_parray(other_array)

                # remove the non local particles
                first_array.remove_tagged_particles(1)

            ret[array_name] = first_array

    else:
        ret = arrays_by_rank[0]

    return ret

# SPH interpolation of data
from pyzoltan.core.carray import UIntArray
from pysph.base.kernels import Gaussian, get_compiled_kernel
from pysph.base.nnps import LinkedListNNPS as NNPS
class SPHInterpolate(object):
    """Class to perform SPH interpolation

    Given solution data on possibly a scattered set, SPHInterpolate
    can be used to interpolate solution data on a regular grid.

    """
    def __init__(self, dim, dst, src, kernel=None):
        self.dst = dst; self.src = src
        if kernel is None:
            self.kernel = get_compiled_kernel(Gaussian(dim))

        # create the neighbor locator object
        self.nnps = nnps = NNPS(dim=dim, particles=[dst, src],
                                radius_scale=self.kernel.radius_scale)
        nnps.update()

    def interpolate(self, arr):
        """Interpolate data given in arr onto coordinate positions"""
        # the result array
        np = self.dst.get_number_of_particles()
        result = numpy.zeros(np)

        nbrs = UIntArray()

        # source arrays
        src = self.src
        sx, sy, sz, sh = src.get('x', 'y', 'z', 'h', only_real_particles=False)

        # dest arrays
        dst = self.dst
        dx, dy, dz, dh = dst.x, dst.y, dst.z, dst.h

        # kernel
        kernel = self.kernel

        for i in range(np):
            self.nnps.get_nearest_particles(src_index=1, dst_index=0, d_idx=i, nbrs=nbrs)
            nnbrs = nbrs.length

            _wij = 0.0; _sum = 0.0
            for indexj in range(nnbrs):
                j  = nbrs[indexj]

                hij = 0.5 * (sh[j] + dh[i])

                wij = kernel.kernel(dx[i], dy[i], dz[i], sx[j], sy[j], sz[j], hij)
                _wij += wij
                _sum += arr[j] * wij

            # save the result
            result[i] = _sum/_wij

        return result

    def gradient(self, arr):
        """Compute the gradient on the interpolated data"""
        # the result array
        np = self.dst.get_number_of_particles()

        # result arrays
        resultx = numpy.zeros(np)
        resulty = numpy.zeros(np)
        resultz = numpy.zeros(np)

        nbrs = UIntArray()

        # data arrays
        # source arrays
        src = self.src
        sx, sy, sz, sh, srho, sm = src.get('x', 'y', 'z', 'h', 'rho', 'm', only_real_particles=False)

        # dest arrays
        dst = self.dst
        dx, dy, dz, dh = dst.x, dst.y, dst.z, dst.h

        # kernel
        kernel = self.kernel

        for i in range(np):
            self.nnps.get_nearest_particles(src_index=1, dst_index=0, d_idx=i, nbrs=nbrs)
            nnbrs = nbrs.length

            _wij = 0.0; _sumx = 0.0; _sumy = 0.0; _sumz = 0.0
            for indexj in range(nnbrs):
                j  = nbrs[indexj]

                hij = 0.5 * (sh[j] + dh[i])

                wij = kernel.kernel(dx[i], dy[i], dz[i], sx[j], sy[j], sz[j], hij)

                rhoj = srho[j]; mj = sm[j]
                fji = arr[j] - arr[i]

                # kernel gradient
                dwij = kernel.gradient(dx[i], dy[i], dz[i], sx[j], sy[j], sz[j], hij)

                _wij += wij
                tmp = 1./rhoj * fji * mj

                _sumx += tmp * dwij.x
                _sumy += tmp * dwij.y
                _sumz += tmp * dwij.z

            # save the result
            resultx[i] = _sumx
            resulty[i] = _sumy
            resultz[i] = _sumz

        return resultx, resulty, resultz

def get_files(dirname=None, fname=None, endswith=output_formats):
    """Get all solution files in a given directory, `dirname`.

    Parameters
    ----------

    dirname: str
        Name of directory.
    fname: str
        An initial part of the filename, if not specified use the first
        part of the dirname.
    endswith: str
        The extension of the file to load.
    """

    if dirname is None:
        return []

    if fname is None:
        fname = dirname.split("_output")[0]

    path = os.path.abspath( dirname )
    files = os.listdir( path )

    # get all the output files in the directory
    files = [f for f in files if f.startswith(fname) and f.endswith(endswith)]
    files = [os.path.join(path, f) for f in files]

    # sort the files
    def _key_func(arg):
        a = os.path.splitext(arg)[0]
        return int(a[a.rfind('_')+1:])

    files.sort(key=_key_func)

    return files


def iter_output(files, *arrays):
    """Given an iterable of the solution files, this loads the files, and
    yields the solver data and the requested arrays.

    If arrays is not supplied, it returns a dictionary of the arrays.

    Parameters
    ----------

    files : iterable
        Iterates over the list of desired files

    *arrays : strings
        Optional series of array names of arrays to return.

    Examples
    --------

    >>> files = get_files('elliptical_drop_output')
    >>> for solver_data, arrays in iter_output(files):
    ...     print(solver_data['t'], arrays.keys())

    >>> files = get_files('elliptical_drop_output')
    >>> for solver_data, fluid in iter_output(files, 'fluid'):
    ...     print(solver_data['t'], fluid.name)

    """
    for file in files:
        data = load(file)
        solver_data = data['solver_data']
        if len(arrays) == 0:
            yield solver_data, data['arrays']
        else:
            _arrays = [data['arrays'][x] for x in arrays]
            yield [solver_data] + _arrays

def _sort_key(arg):
    a = os.path.splitext(arg)[0]
    return int(a[a.rfind('_')+1:])

def remove_irrelevant_files(files):
    """Remove any npz files that are not output files.

    That is, the file should not end with a '_number.npz'.  This allows users
    to dump other .npz of .hdf5 files in the output while post-processing without
    breaking.
    """
    result = []
    for f in files:
        try:
            _sort_key(f)
        except ValueError:
            pass
        else:
            result.append(f)
    return result




# Standard imports.
import glob
import inspect
import json
import logging
import os
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from os.path import (abspath, basename, dirname, isdir, join, realpath,
    splitext)
import sys
import time

# PySPH imports.
from pysph.base.config import get_config
from pysph.base import utils
from pysph.base.nnps import BoxSortNNPS, LinkedListNNPS
from pysph.base import kernels
from pysph.solver.controller import CommandManager
from pysph.solver.utils import mkdir, load, get_files

# conditional parallel imports
from pysph import has_mpi, has_zoltan, in_parallel
if in_parallel():
    from pysph.parallel.parallel_manager import ZoltanParallelManagerGeometric
    import mpi4py.MPI as mpi

logger = logging.getLogger(__name__)


def is_overloaded_method(method):
    """Returns True if the given method is overloaded from any of its bases.
    """
    method_name = method.__name__
    self = method.__self__
    klass = self.__class__
    for base in klass.__bases__:
        if hasattr(base, method_name):
            if getattr(base, method_name) != getattr(klass, method_name):
                return True
    return False

def is_using_ipython():
    """Return True if the code is being run from an IPython session or
    notebook.
    """
    try:
        # If this is being run inside an IPython console or notebook
        # then this is defined.
        __IPYTHON__
    except NameError:
        return False
    else:
        return True

def list_all_kernels():
    """Return list of available kernels.
    """
    return [n for n in dir(kernels) if inspect.isclass(getattr(kernels, n))]


##############################################################################
# `Application` class.
##############################################################################
class Application(object):
    """ Class used by any SPH application.
    """

    def __init__(self, fname=None, domain=None):
        """ Constructor

        Parameters
        ----------
        fname : str
            file name to use for the output files.
        domain : pysph.nnps.DomainManager
            A domain manager to use. This is used for periodic domains etc.
        """
        self.domain = domain

        self.solver = None
        self.nnps = None
        self.scheme = None
        self.tools = []
        self.parallel_manager = None

        if fname is None:
            fname = self._guess_output_filename()

        self.fname = fname

        self.args = sys.argv[1:]

        # MPI related vars.
        self.comm = None
        self.num_procs = 1
        self.rank = 0
        if in_parallel():
            self.comm = comm = mpi.COMM_WORLD
            self.num_procs = comm.Get_size()
            self.rank = comm.Get_rank()

        self._log_levels = {'debug': logging.DEBUG,
                           'info': logging.INFO,
                           'warning': logging.WARNING,
                           'error': logging.ERROR,
                           'critical': logging.CRITICAL,
                           'none': None}

        self.output_dir = abspath(self._get_output_dir_from_fname())
        self.particles = []
        self.inlet_outlet = []

        self.initialize()
        self.scheme = self.create_scheme()
        self._setup_argparse()

    def _get_output_dir_from_fname(self):
        return self.fname + '_output'

    def _guess_output_filename(self):
        """Try to guess the output filename to use.
        """
        module = self.__module__.rsplit('.', 1)[-1]
        if is_using_ipython():
            return module
        else:
            if len(sys.argv[0]) == 0:
                return module
            else:
                return splitext(basename(abspath(sys.argv[0])))[0]

    def _setup_argparse(self):
        usage = '%(prog)s [options]'
        description = """
        Note that you may run this program via MPI and the run will be
        automatically parallelized.  To do this run::

         $ mpirun -n 4 /path/to/your/python %prog [options]

        Replace '4' above with the number of processors you have.
        Below are the options you may pass.

        """
        parser = ArgumentParser(
            usage=usage, description=description,
            formatter_class=ArgumentDefaultsHelpFormatter
        )
        self.arg_parse = parser

         # Add some default options.
         # -v
        valid_vals = "Valid values: %s"%self._log_levels.keys()
        parser.add_argument("-v", "--loglevel", action="store",
                          dest="loglevel",
                          default='info',
                          help="Log-level to use for log messages. " +
                               valid_vals)
        # --logfile

        parser.add_argument("--logfile", action="store",
                          dest="logfile",
                          default=None,
                          help="Log file to use for logging, set to "+
                               "empty ('') for no file logging.")
        # -l
        parser.add_argument("-l", "--print-log", action="store_true",
                          dest="print_log", default=False,
                          help="Print log messages to stderr.")
        # --final-time
        parser.add_argument("--tf", action="store",
                          type=float,
                          dest="final_time",
                          default=None,
                          help="Total time for the simulation.")
        # --timestep
        parser.add_argument("--timestep", action="store",
                          type=float,
                          dest="time_step",
                          default=None,
                          help="Timestep to use for the simulation.")
        # --max-steps
        parser.add_argument(
            "--max-steps", action="store", type=int, dest="max_steps",
            default=1<<31,
            help="Maximum number of iteration steps to take (defaults to a "
            "very large value)."
        )

        # --adaptive-timestep
        parser.add_argument("--adaptive-timestep", action="store_true",
                          dest="adaptive_timestep", default=None,
                          help="Use adaptive time stepping.")
        parser.add_argument("--no-adaptive-timestep", action="store_false",
                          dest="adaptive_timestep", default=None,
                          help="Do not use adaptive time stepping.")

        # --cfl
        parser.add_argument("--cfl", action="store", dest="cfl", type=float,
                          default=0.3,
                          help="CFL number for adaptive time steps")

        # -q/--quiet.
        parser.add_argument("-q", "--quiet", action="store_true",
                         dest="quiet", default=False,
                         help="Do not print any progress information.")

        # --disable-output
        parser.add_argument("--disable-output", action="store_true",
                         dest="disable_output", default=False,
                         help="Do not dump any output files.")

        # -o/ --fname
        parser.add_argument("-o", "--fname", action="store",
                          dest="fname", default=self.fname,
                          help="File name to use for output")

        # --pfreq.
        parser.add_argument("--pfreq", action="store",
                          dest="freq", default=None, type=int,
                          help="Printing frequency for the output")

        # --detailed-output.
        parser.add_argument("--detailed-output", action="store_true",
                         dest="detailed_output", default=None,
                         help="Dump detailed output.")

        # -z/--compress-output
        parser.add_argument(
            "-z", "--compress-output", action="store_true",
            dest="compress_output", default=False,
            help="Compress generated output files."
        )

        # --output-remote
        parser.add_argument("--output-dump-remote", action="store_true",
                          dest="output_dump_remote", default=False,
                          help="Save Remote particles in parallel")
        # -d/--directory
        parser.add_argument("-d", "--directory", action="store",
                          dest="output_dir",
                          default=self._get_output_dir_from_fname(),
                          help="Dump output in the specified directory.")

        # --openmp
        parser.add_argument("--openmp", action="store_true", dest="with_openmp",
                          default=None, help="Use OpenMP to run the "\
                            "simulation using multiple cores.")
        parser.add_argument("--no-openmp", action="store_false", dest="with_openmp",
                          default=None, help="Do not use OpenMP to run the "\
                            "simulation using multiple cores.")

        # --kernel
        all_kernels = list_all_kernels()
        parser.add_argument(
            "--kernel", action="store", dest="kernel", default=None,
             choices=all_kernels,
            help="Use specified kernel from %s"%all_kernels
        )

        # Restart options
        restart = parser.add_argument_group("Restart options",
                              "Restart options for PySPH")

        restart.add_argument("--restart-file", action="store", dest="restart_file",
                           default=None,
                           help=("""Restart a PySPH simulation using a specified file """)),

        restart.add_argument("--rescale-dt", action="store", dest="rescale_dt",
                           default=1.0, type=float,
                           help=("Scale dt upon restarting by a numerical constant"))


        # NNPS options
        nnps_options = parser.add_argument_group("NNPS",
                "Nearest Neighbor searching")

        # --nnps
        nnps_options.add_argument("--nnps", dest="nnps",
                                choices=['box', 'll'],
                                default='ll',
                                help="Use one of box-sort ('box') or "\
                                     "the linked list algorithm ('ll'). "
                                )

        # --fixed-h
        nnps_options.add_argument("--fixed-h", dest="fixed_h",
                                action="store_true", default=False,
                                help="Option for fixed smoothing lengths")

        nnps_options.add_argument("--cache-nnps", dest="cache_nnps",
                                action="store_true", default=False,
                        help="Option to enable the use of neighbor caching.")

        nnps_options.add_argument(
            "--sort-gids", dest="sort_gids", action="store_true",
            default=False, help="Sort neighbors by the GIDs to get "\
            "consistent results in serial and parallel (slows down a bit)."
        )


        # Zoltan Options
        zoltan = parser.add_argument_group("PyZoltan",
                             "Zoltan load balancing options")

        zoltan.add_argument("--with-zoltan", action="store_true",
                          dest="with_zoltan", default=True,
                          help=("""Use PyZoltan for dynamic load balancing """))

        zoltan.add_argument("--zoltan-lb-method", action="store",
                          dest="zoltan_lb_method", default="RCB",
                          help=("""Choose the Zoltan load balancnig method"""))

        # --rcb-lock
        zoltan.add_argument("--rcb-lock", action="store_true", dest="zoltan_rcb_lock_directions",
                          default=False,
                          help=("Lock the directions of the RCB cuts"))

        # rcb--reuse
        zoltan.add_argument("--rcb-reuse", action='store_true', dest="zoltan_rcb_reuse",
                          default=False,
                          help=("Reuse previous RCB cuts"))

        # rcb-rectilinear
        zoltan.add_argument("--rcb-rectilinear", action="store_true", dest='zoltan_rcb_rectilinear',
                          default=False,
                          help=("Produce nice rectilinear blocks without projections"))

        # rcb-set-direction
        zoltan.add_argument("--rcb-set-direction", action='store', dest="zoltan_rcb_set_direction",
                          default=0, type=int,
                          help=("Set the order of the RCB cuts"))

        zoltan.add_argument("--zoltan-weights", action="store_false",
                          dest="zoltan_weights", default=True,
                          help=("""Switch between using weights for input to Zoltan.
                          defaults to True"""))

        zoltan.add_argument("--ghost-layers", action='store', dest='ghost_layers',
                          default=3.0, type=float,
                          help=('Number of ghost cells to share for remote neighbors'))

        zoltan.add_argument("--lb-freq", action='store', dest='lb_freq',
                          default=10, type=int,
                          help=('The frequency for load balancing'))

        zoltan.add_argument("--zoltan-debug-level", action="store",
                          dest="zoltan_debug_level", default="0",
                          help=("""Zoltan debugging level"""))


        # Options to control parallel execution
        parallel_options=parser.add_argument_group("Parallel Options")

        # --update-cell-sizes
        parallel_options.add_argument("--update-cell-sizes", action='store_true',
                                    dest='update_cell_sizes', default=False,
                                    help=("Recompute cell sizes for binning in parallel"))

        # --parallel-scale-factor
        parallel_options.add_argument("--parallel-scale-factor", action="store",
                                    dest="parallel_scale_factor", default=2.0, type=float,
                                    help=("""Kernel scale factor for the parallel update"""))

        # --parallel-output-mode
        parallel_options.add_argument("--parallel-output-mode", action="store",
                            dest="parallel_output_mode", default=None,
                            help="""Use 'collected' to dump one output at
                          root or 'distributed' for every processor. """)



        # solver interfaces
        interfaces = parser.add_argument_group("Interfaces",
                                 "Add interfaces to the solver")

        interfaces.add_argument("--interactive", action="store_true",
                              dest="cmd_line", default=False,
                              help=("Add an interactive commandline interface "
                                    "to the solver"))

        interfaces.add_argument("--xml-rpc", action="store",
                dest="xml_rpc", metavar="[HOST:] PORT",
                              help=("Add an XML-RPC interface to the solver;"
                                      "HOST=0.0.0.0 by default"))

        interfaces.add_argument("--multiproc", action="store",
                              dest="multiproc", metavar='[[AUTHKEY@] HOST:] PORT[+] ',
                              default="pysph@0.0.0.0:8800+",
                              help=("Add a python multiprocessing interface "
                                    "to the solver; "
                                    "AUTHKEY=pysph, HOST=0.0.0.0, PORT=8800+ by"
                                    " default (8800+ means first available port "
                                    "number 8800 onwards)"))

        interfaces.add_argument("--no-multiproc", action="store_const",
                              dest="multiproc", const=None,
                              help=("Disable multiprocessing interface "
                                    "to the solver"))

          # User options.
        user_options = parser.add_argument_group("User",
                 "User defined command line arguments")
        if self.scheme is not None:
            self.scheme.add_user_options(user_options)
        self.add_user_options(user_options)

    def _parse_command_line(self, force=False):
        """If force is True, it will parse the arguments regardless of whether
        it is running in IPython or not.  This is handy when you want to parse
        the command line for a previously run case.
        """
        if is_using_ipython() and not force:
            # Don't parse the command line args.
            options  = self.arg_parse.parse_args([])
        else:
            options  = self.arg_parse.parse_args(self.args)

        self.options = options

        # save the path where we want to dump output
        self.output_dir = abspath(options.output_dir)
        mkdir(self.output_dir)
        if self.scheme is not None:
            self.scheme.consume_user_options(self.options)
        self.consume_user_options()
        self.configure_scheme()

    def _setup_logging(self):
        """Setup logging for the application.
        """
        options = self.options
        # Setup logging based on command line options.
        level = self._log_levels[options.loglevel]

        if level is None:
            return

        # logging setup
        logger.setLevel(level)

        filename = options.logfile
        # Setup the log file.
        if filename is None:
            filename = self.fname + '.log'

        if len(filename) > 0:
            lfn = os.path.join(self.output_dir,filename)
            format = '%(levelname)s|%(asctime)s|%(name)s|%(message)s'
            logging.basicConfig(level=level, format=format,
                                filename=lfn, filemode='a')
        if options.print_log:
            logger.addHandler(logging.StreamHandler())

    def _create_inlet_outlet(self, inlet_outlet_factory):
        """Create the inlets and outlets if needed.

        This method requires that the particles be already created.

        The `inlet_outlet_factory` is passed a dictionary of the particle
        arrays.  The factory should return a list of inlets and outlets.
        """
        if inlet_outlet_factory is not None:
            solver = self.solver
            particle_arrays = dict([(p.name, p) for p in self.particles])
            self.inlet_outlet = inlet_outlet_factory(particle_arrays)
            # Hook up the inlet/outlet's update method to be called after
            # each stage.
            for obj in self.inlet_outlet:
                solver.add_post_step_callback(obj.update)

    def _create_particles(self, particle_factory, *args, **kw):
        """ Create particles given a callable `particle_factory` and any
        arguments to it.
        """
        options = self.options
        rank = self.rank

        # particle array info that is used to create dummy particles
        # on non-root processors
        particles_info = {}

        # Only master actually calls the particle factory, the rest create
        # dummy particle arrays.
        if rank == 0:
            if options.restart_file is not None:
                # FIXME: not tested, probably does not work!
                solver = self.solver
                data = load(options.restart_file)

                arrays = data['arrays']
                solver_data = data['solver_data']

                # arrays and particles
                particles = []
                for array_name in arrays:
                    particles.append( arrays[array_name] )

                # save the particles list
                self.particles = particles

                # time, timestep and solver iteration count at restart
                t, dt, count = solver_data['t'], solver_data['dt'], solver_data['count']

                # rescale dt at restart
                dt *= options.rescale_dt
                solver.t, solver.dt, solver.count  = t, dt, count

            else:
                self.particles = particle_factory(*args, **kw)

            # get the array info which will be b'casted to other procs
            particles_info = utils.get_particles_info(self.particles)

        # Broadcast the particles_info to other processors for parallel runs
        if self.num_procs > 1:
            particles_info = self.comm.bcast(particles_info, root=0)

        # now all processors other than root create dummy particle arrays
        if rank != 0:
            self.particles = utils.create_dummy_particles(particles_info)

    def _configure(self):
        """Configures the application using the options from the
        command-line.
        """
        options = self.options
        # Setup configuration options.
        if options.with_openmp is not None:
            get_config().use_openmp = options.with_openmp
        # setup the solver using any options
        self.solver.setup_solver(options.__dict__)

        solver = self.solver

        # fixed smoothing lengths
        fixed_h = solver.fixed_h or options.fixed_h

        kernel = solver.kernel
        if options.kernel is not None:
            kernel = getattr(kernels, options.kernel)(dim=solver.dim)
            solver.kernel = kernel

        # This should be called before an NNPS is created as the particles are
        # changed after the initial load-balancing.
        self._setup_parallel_manager_and_initial_load_balance()

        if self.nnps is None:
            cache = options.cache_nnps

            # create the NNPS object
            if options.nnps == 'box':
                nnps = BoxSortNNPS(
                    dim=solver.dim, particles=self.particles,
                    radius_scale=kernel.radius_scale, domain=self.domain,
                    cache=cache, sort_gids=options.sort_gids
                )

            elif options.nnps == 'll':
                nnps = LinkedListNNPS(
                    dim=solver.dim, particles=self.particles,
                    radius_scale=kernel.radius_scale, domain=self.domain,
                    fixed_h=fixed_h, cache=cache,
                    sort_gids=options.sort_gids
                )
            self.nnps = nnps

        nnps = self.nnps
        # once the NNPS has been set-up, we set the default Solver
        # post-stage callback to the DomainManager.setup_domain
        # method. This method is responsible to computing the new cell
        # size and doing any periodicity checks if needed.
        solver.add_post_stage_callback( nnps.update_domain )

        # inform NNPS if it's working in parallel
        if self.num_procs > 1:
            nnps.set_in_parallel(True)

        dt = options.time_step
        if dt is not None:
            solver.set_time_step(dt)

        tf = options.final_time
        if tf is not None:
            solver.set_final_time(tf)

        solver.set_max_steps(self.options.max_steps)

        # Setup the solver output file name
        fname = options.fname

        if in_parallel():
            rank = self.rank
            if self.num_procs > 1:
                fname += '_' + str(rank)

        # set the rank for the solver
        solver.rank = self.rank
        solver.pid = self.rank
        solver.comm = self.comm

        # set the in parallel flag for the solver
        if self.num_procs > 1:
            solver.in_parallel = True

        # output file name
        solver.set_output_fname(fname)

        solver.set_compress_output(options.compress_output)
        # disable_output
        solver.set_disable_output(options.disable_output)

        # output print frequency
        if options.freq is not None:
            solver.set_print_freq(options.freq)

        # output printing level (default is not detailed)
        if options.detailed_output is not None:
            solver.set_output_printing_level(options.detailed_output)

        # solver output behaviour in parallel
        if options.output_dump_remote:
            solver.set_output_only_real( False )

        # output directory
        solver.set_output_directory(abspath(options.output_dir))
        self._message("Generating output in %s"%self.output_dir)

        # set parallel output mode
        if options.parallel_output_mode is not None:
            solver.set_parallel_output_mode(options.parallel_output_mode)

        # Set the adaptive timestep
        if options.adaptive_timestep is not None:
            solver.set_adaptive_timestep(options.adaptive_timestep)

            # set solver cfl number
            solver.set_cfl(options.cfl)


        # setup the solver. This is where the code is compiled
        solver.setup(
            particles=self.particles, equations=self.equations, nnps=nnps,
            kernel=kernel, fixed_h=fixed_h
        )

        # add solver interfaces
        self.command_manager = CommandManager(solver, self.comm)
        solver.set_command_handler(self.command_manager.execute_commands)

        if self.rank == 0:
            # commandline interface
            if options.cmd_line:
                from pysph.solver.solver_interfaces import CommandlineInterface
                self.command_manager.add_interface(CommandlineInterface().start)

            # XML-RPC interface
            if options.xml_rpc:
                from pysph.solver.solver_interfaces import XMLRPCInterface
                addr = options.xml_rpc
                idx = addr.find(':')
                host = "0.0.0.0" if idx == -1 else addr[:idx]
                port = int(addr[idx+1:])
                self.command_manager.add_interface(XMLRPCInterface((host,port)).start)

            # python MultiProcessing interface
            if options.multiproc:
                from pysph.solver.solver_interfaces import MultiprocessingInterface
                addr = options.multiproc
                idx = addr.find('@')
                authkey = "pysph" if idx == -1 else addr[:idx]
                addr = addr[idx+1:]
                idx = addr.find(':')
                host = "0.0.0.0" if idx == -1 else addr[:idx]
                port = addr[idx+1:]
                if port[-1] == '+':
                    try_next_port = True
                    port = port[:-1]
                else:
                    try_next_port = False
                port = int(port)

                interface = MultiprocessingInterface(
                    (host,port), authkey.encode(), try_next_port)

                self.command_manager.add_interface(interface.start)

                logger.info('started multiprocessing interface on %s'%(
                             interface.address,))

    def _setup_parallel_manager_and_initial_load_balance(self):
        """This will automatically distribute the particles among processors
        if this is a parallel run.
        """
        # Instantiate the Parallel Manager here and do an initial LB
        num_procs = self.num_procs
        options = self.options
        solver = self.solver
        comm = self.comm

        self.parallel_manager = None
        if num_procs > 1:
            options = self.options

            if options.with_zoltan:
                if not (has_zoltan() and has_mpi()):
                    raise RuntimeError("Cannot run in parallel!")

            else:
                raise ValueError("""Sorry. You're stuck with Zoltan for now
                use the option '--with-zoltan' for parallel runs
                """)

            # create the parallel manager
            obj_weight_dim = "0"
            if options.zoltan_weights:
                obj_weight_dim = "1"

            zoltan_lb_method = options.zoltan_lb_method
            zoltan_obj_wgt_dim = obj_weight_dim

            # ghost layers
            ghost_layers = options.ghost_layers

            # radius scale for the parallel update
            radius_scale = options.parallel_scale_factor*solver.kernel.radius_scale

            self.parallel_manager = pm = ZoltanParallelManagerGeometric(
                dim=solver.dim, particles=self.particles, comm=comm,
                lb_method=zoltan_lb_method,
                obj_weight_dim=obj_weight_dim,
                ghost_layers=ghost_layers,
                update_cell_sizes=options.update_cell_sizes,
                radius_scale=radius_scale,
                )

            ### ADDITIONAL LOAD BALANCING FUNCTIONS FOR ZOLTAN ###

            # RCB lock directions
            if options.zoltan_rcb_lock_directions:
                pm.set_zoltan_rcb_lock_directions()

            if options.zoltan_rcb_reuse:
                pm.set_zoltan_rcb_reuse()

            if options.zoltan_rcb_rectilinear:
                pm.set_zoltan_rcb_rectilinear_blocks()

            if options.zoltan_rcb_set_direction > 0:
                pm.set_zoltan_rcb_directions( str(options.zoltan_rcb_set_direction) )

            # set zoltan options
            pm.pz.Zoltan_Set_Param("DEBUG_LEVEL", options.zoltan_debug_level)
            pm.pz.Zoltan_Set_Param("DEBUG_MEMORY", "0")

            # do an initial load balance
            pm.update()
            pm.initial_update = False

            # set subsequent load balancing frequency
            lb_freq = options.lb_freq
            if lb_freq < 1 : raise ValueError("Invalid lb_freq %d"%lb_freq)
            pm.set_lb_freq( lb_freq )

            # wait till the initial partition is done
            comm.barrier()

        # set the solver's parallel manager
        solver.set_parallel_manager(self.parallel_manager)

    def _setup_solver_callbacks(self, obj):
        """Setup any solver callbacks given an object with any of `pre_step`,
        `post_step' and `post_stage`
        """
        if is_overloaded_method(obj.pre_step):
            self.solver.add_pre_step_callback(obj.pre_step)

        if is_overloaded_method(obj.post_stage):
            self.solver.add_post_stage_callback(obj.post_stage)

        if is_overloaded_method(obj.post_step):
            self.solver.add_post_step_callback(obj.post_step)

    def _message(self, msg):
        if self.options.quiet:
            return
        if self.num_procs == 1:
            logger.info(msg)
            print(msg)
        elif (self.num_procs > 1 and self.rank in (0,1)):
            s = "Rank %d: %s"%(self.rank, msg)
            logger.info(s)
            print(s)

    def _write_info(self, filename, **kw):
        """Write the information dictionary to given filename. Any extra
        keyword arguments are written to the file.
        """
        info = dict(
            fname=self.fname, output_dir=self.output_dir, args=self.args
        )
        info.update(kw)
        json.dump(info, open(filename, 'w'))

    ######################################################################
    # Public interface.
    ######################################################################
    def add_tool(self, tool):
        """Add a `Tool` instance to the application.
        """
        self._setup_solver_callbacks(tool)
        self.tools.append(tool)

    def dump_code(self, file):
        """Dump the generated code to given file.
        """
        file.write(self.solver.sph_eval.ext_mod.code)

    @property
    def info_filename(self):
        return abspath(join(self.output_dir, self.fname + '.info'))

    def initialize(self):
        """Called on the constructor, set constants etc. up here if needed.
        """
        pass

    @property
    def output_files(self):
        return get_files(self.output_dir, self.fname)

    def read_info(self, fname_or_dir):
        """Read the information from the given info file (or directory
        containing the info file, the first found info file will be used).
        """
        if isdir(fname_or_dir):
            fname_or_dir = glob.glob(join(fname_or_dir, "*.info"))[0]
        info_dir = dirname(fname_or_dir)
        with open(fname_or_dir, 'r') as f:
            info = json.load(f)
        self.fname = info.get('fname', self.fname)
        output_dir = info.get('output_dir', self.output_dir)
        if realpath(info_dir) != realpath(output_dir):
            # Happens if someone moved the directory!
            self.output_dir = info_dir
            info['output_dir'] = info_dir
        else:
            self.output_dir = output_dir

        self.args = info.get('args', self.args)
        self._parse_command_line(force=True)
        return info

    def run(self, argv=None):
        """Run the application.
        """
        if argv is not None:
            self.set_args(argv)

        if self.solver is None:
            start_time = time.time()

            self._parse_command_line()
            self._setup_logging()

            self.solver = self.create_solver()
            msg = "Solver is None, you may have forgotten to return it!"
            assert self.solver is not None, msg
            self.equations = self.create_equations()

            self._create_particles(self.create_particles)

            # This must be done before the initial load balancing
            # as the inlets will create new particles.
            if is_overloaded_method(self.create_inlet_outlet):
                self._create_inlet_outlet(self.create_inlet_outlet)

            if self.domain is None:
                self.domain = self.create_domain()

            self.nnps = self.create_nnps()

            self._configure()

            self._setup_solver_callbacks(self)
            for tool in self.create_tools():
                self.add_tool(tool)

            end_time = time.time()
            setup_duration = end_time - start_time
            self._message("Setup took: %.5f secs"%(setup_duration))
            self._write_info(
                self.info_filename, completed=False, cpu_time=0
            )

        start_time = time.time()
        self.solver.solve(not self.options.quiet)
        end_time = time.time()
        run_duration = end_time - start_time
        self._message("Run took: %.5f secs"%(run_duration))
        self._write_info(
            self.info_filename, completed=True, cpu_time=run_duration
        )

    def set_args(self, args):
        self.args = args

    def setup(self, solver, equations, nnps=None, inlet_outlet_factory=None,
              particle_factory=None, *args, **kwargs):
        """Setup the application's solver.

        This will parse the command line arguments (if this is not called from
        within an IPython notebook or shell) and then using those parameters
        and any additional parameters and call the solver's setup method.

        Parameters
        ----------
        solver: pysph.solver.solver.Solver
            The solver instance.

        equations: list
            A list of Groups/Equations.

        nnps: pysph.base.nnps.NNPS
            Optional NNPS instance. If None is given a default NNPS is created.

        inlet_outlet_factory: callable or None
            The `inlet_outlet_factory` is passed a dictionary of the particle
            arrays.  The factory should return a list of inlets and outlets.

        particle_factory : callable or None
            If supplied, particles will be created for the solver using the
            particle arrays returned by the callable. Else particles for the
            solver need to be set before calling this method

        args:
            extra positional arguments passed on to the `particle_factory`.

        kwargs:
            extra keyword arguments passed to the `particle_factory`.


        Examples
        --------

        >>> def create_particles():
        ...    ...
        ...
        >>> solver = Solver(...)
        >>> equations = [...]
        >>> app = Application()
        >>> app.setup(solver=solver, equations=equations,
        ...           particle_factory=create_particles)
        >>> app.run()
        """
        start_time = time.time()
        self.solver = solver
        self.equations = equations
        solver_opts = solver.get_options(self.arg_parse)
        self._parse_command_line()
        self._setup_logging()

        # Create particles either from scratch or restart
        self._create_particles(particle_factory, *args, **kwargs)

        # This must be done before the initial load balancing
        # as the inlets will create new particles.
        self._create_inlet_outlet(inlet_outlet_factory)
        if nnps is not None:
            self.nnps = nnps

        self._configure()

        end_time = time.time()
        setup_duration = end_time - start_time
        self._message("Setup took: %.5f secs"%(setup_duration))
        self._write_info(self.info_filename, completed=False, cpu_time=0)

    ######################################################################
    # User methods that could be overloaded.
    ######################################################################
    def add_user_options(self, group):
        """Add any user-defined options to the given option group.

        Note
        ----

        This uses the `argparse` module.
        """
        pass

    def configure_scheme(self):
        """This is called after ``consume_user_options`` is called.  One can
        configure the SPH scheme here as at this point all the command line
        options are known.
        """
        pass

    def consume_user_options(self):
        """This is called right after the command line arguments are parsed.

        All the parsed options are available in ``self.options`` and can be
        used in this method.

        This is meant to be overridden by users to setup any internal variables
        etc. that depend on the command line arguments passed.  Note that this
        method is called well before the solver or particles are created.
        """
        pass

    def create_domain(self):
        """Create a `pysph.nnps.DomainManager` and return it if needed.

        This is used for periodic domains etc.  Note that if the domain
        is passed to ``__init__``, then this method is not called.
        """
        return None

    def create_inlet_outlet(self, particle_arrays):
        """Create inlet and outlet objects and return them as a list.

        The method is passed a dictionary of particle arrays keyed on the name
        of the particle array.
        """
        pass

    def create_equations(self):
        """Create the equations to be used and return them.
        """
        if self.scheme is not None:
            return self.scheme.get_equations()
        else:
            msg = "Application.create_equations method must be overloaded."
            raise NotImplementedError(msg)

    def create_nnps(self):
        """Create any NNPS if desired and return it, else a default NNPS will
        be created automatically.
        """
        return None

    def create_particles(self):
        """Create particle arrays and return a list of them.
        """
        message = "Application.create_particles method must be overloaded."
        raise NotImplementedError(message)

    def create_scheme(self):
        """Create a suitable SPH scheme and return it.

        Note that this method is called after the arguments are all
        processed and after `consume_user_options` is called.
        """
        return None

    def create_solver(self):
        """Create the solver and return it.
        """
        if self.scheme is not None:
            return self.scheme.get_solver()
        else:
            msg = "Application.create_solver method must be overloaded."
            raise NotImplementedError(msg)

    def create_tools(self):
        """Create any tools and return a sequence of them.  This method is
        called after particles/inlets etc. are all setup, configured etc.
        """
        return []

    def pre_step(self, solver):
        """If overloaded, this is called automatically before each integrator
        step.  The method is passed the solver instance.
        """
        pass

    def post_stage(self, current_time, dt, stage):
        """If overloaded, this is called automatically after each integrator
        stage, i.e. if the integrator is a two stage integrator it will be
        called after the first and second stages.

        The method is passed (current_time, dt, stage).  See the the
        `Integrator.one_timestep` methods for examples of how this is called.
        """
        pass

    def post_step(self, solver):
        """If overloaded, this is called automatically after each integrator
        step.  The method is passed the solver instance.
        """
        pass

    def post_process(self, info_fname_or_directory):
        """Given an info filename or a directory containing the info file, read
        the information and do any post-processing of the results.  Please
        overload the method to perform any processing.

        The info file has a few useful attributes and can be read using the
        `read_info` method.

        The `output_files` property should provide the output files
        generated.
        """
        print('Overload this method to post-process the results.')
